#ifndef LEAF_BIN_EXPECTER_HPP
#define LEAF_BIN_EXPECTER_HPP

#include "../core_types.hpp"

#include <boost/optional.hpp>

#include "runner/expecter.hpp"

/**
	Loads test conditions from source files and checks their results. Used for unit testing
	of Leaf.
*/
typedef boost::error_info<struct tag_expect,std::string> error_expect;
typedef boost::error_info<struct tag_actual,std::string> error_actual;

struct test_expecter : expecter {
	bool omit, ignore;
	
	struct expect_fail {
		bool set; //if true then expect this failure
		std::string what; //if empty then any generic error
		operator bool() const { 
			return set; 
		}
	} expect_fail_typer, expect_fail_converter, expect_fail_parser, expect_fail_exec;
	std::string src;
	boost::optional<unsigned> src_line;
	boost::optional<int> expect_return_value;
	boost::optional<std::string> expect_output;
	
	/**
		@param src the primary source code to scan for conditions.
	*/
	test_expecter( std::string const & src );
	
	/**
		@return true if there are any conditions
	*/
	bool any_expect() const {
		return expect_output || expect_return_value || any_expect_fail();
	}
	
	/**
		Check the execution result against conditions.
		
		Returns if okay, throws on error.
	*/
	void check_result( std::string const & actual, int return_value ) override;
	
	/**
		Check if the compile-time exception matches the expected conditions.
		
		Returns if the exception was properly caught, rethrows on unknown/mismatch.
	*/
	void catch_compile_exception( recoverable_error & re ) const override;
	
	/**
		Check an abort message (indicates abort actually occurred).
		
		@return true if message handled, false if unknown error
	*/
	bool catch_runtime_abort( std::string const & e ) const override;
private:
	bool any_expect_fail() const;
	void check_marker( std::string const & text, expect_fail & variable );
};

#endif
