#ifndef LEAF_IR_BLOCK_HPP
#define LEAF_IR_BLOCK_HPP

#include "fwd.hpp"

#include <vector>

namespace ir {

struct block_seq_item {
	//one of these items is defined
	shared_ptr<instruction> an_instruction;
	shared_ptr<block> a_block;
	
	block_flow flow;
	
	block_seq_item( shared_ptr<instruction> & i, block_flow flow ) : 
		an_instruction( i ), 
		flow( flow ) { 
	}
	block_seq_item( shared_ptr<block> & b,  block_flow flow ) : 
		a_block( b ), 
		flow( flow ) { 
	}
};

class block {
	enum private_ctor_t { private_ctor };
	
	block * parent; //may be null
	function * owner_function;
public:
	function & get_owner_function() const;
	
	std::vector<block_seq_item> items;	
	
	//will be null if there is no error_block
	shared_ptr<block> error_block;
	int error_level;
	boost::optional<value_ref> error_info;
	
	void push_back( shared_ptr<instruction> i, block_flow flow ) { 
		items.push_back( block_seq_item( i, flow ) ); 
	}
	void push_back( shared_ptr<block> b, block_flow flow ) { 
		items.push_back( block_seq_item( b, flow ) ); 
	}

	void append( block & o );
	
	block( function * owner, block * parent, private_ctor_t ) : 
		parent( parent ), 
		owner_function( owner ) {
	}
	
	boost::optional<value_ref> find_error_info() const;

	//creates an inserts a child block into the `items` sequence
	shared_ptr<block> insert_block() {
		auto s = make_shared<block>(nullptr, this, private_ctor );
		push_back( s, block_flow::sequential );
		return s;
	}
	//creates a child block but does not insert into the sequence
	shared_ptr<block> create_child_block() {
		return make_shared<block>(nullptr, this, private_ctor );
	}
	
	static shared_ptr<block> create_root( function * owner_function ) {
		STATE_CHECK( owner_function );
		return make_shared<block>(owner_function, nullptr, private_ctor );
	}
	
	shared_ptr<value> make_temp( shared_ptr<type const> dt );
	
	void validate( util::validation_context & vctx ) const;
};

} //eon ir

#endif