#ifndef LEAF_IR_COMPILER_HPP
#define LEAF_IR_COMPILER_HPP

#include "compiler_context.hpp"

#include <deque>
#include <vector>

namespace ir
{

struct module_builder;
struct function_builder;
struct builder;

class compiler : public compiler_context
{
	friend class type_manager;
	
	virtual module & get_cur_module() {
		return *cur_module;
	}
public:
	
protected:
	compiler();
	
	typedef std::vector<value_ref> value_ref_vector_t;
	
	shared_ptr<module> cur_module;
	void init_module( std::string const & name );
	
	void assign( value_ref left, value_ref right, builder & where, builder & store, unsigned flags = 0 );
	void assign( value_ref_vector_t & left, value_ref_vector_t & right, builder & where, unsigned flags = 0 );
	void assign_copy_value( value_ref dst, value_ref src, builder & where, unsigned flags );
	
	bool acquire( value_ref value, builder & where );
	/*override*/ void release( value_ref value, builder & where, block_flow flow, unsigned flags = 0);
	/*override*/ void release_pointer( value_ref ptr, builder & where, block_flow flow, unsigned flags = 0);
	/*override*/ boost::optional<value_ref> get_release_pointer( value_ref value, builder & where, unsigned flags =  0 );
	
	value_ref assign_arg( value_ref in, builder & where );

	virtual value_ref initialize( value_ref ptr, shared_ptr<type const> init_type, 
		unsigned flags, builder & where );
		
	value_ref gen_default( shared_ptr<type const> t, builder & where );
	value_ref gen_shared( shared_ptr<type const> t, builder & where );

	value_ref gen_static_binary_data( uint8_t const * data, size_t len, builder & where );
	
	struct symbol_cref {
		//TODO: something isn't right here, ref has an origin...
		shared_ptr<value> origin;
		value_ref ref;
		//prevent deinit
		bool no_release;
		//is the definition complete
		bool built;
		
		symbol_cref() : no_release(false),
			built( false ) {
		}
	};
	typedef std::deque<symbol_cref> symbol_deque_t;
	symbol_deque_t symbol_deque;
	symbol_cref & get_symbol_cref( intptr_t & cref );
	void define_symbol_cref( intptr_t & cref, value_ref vr );
	
	void configure_destructor( shared_object_defn & sod, builder & where );
};

} //eon ir
#endif