#ifndef LEAF_IR_COMPILER_CONTEXT_HPP
#define LEAF_IR_COMPILER_CONTEXT_HPP

#include "fwd.hpp"

namespace ir
{

struct builder;

struct compiler_context 
{
	virtual module & get_cur_module() = 0;
	virtual type_manager & get_types() = 0;
	
	enum assign_flags
	{
		af_init_refer = 0x1,
		af_force_resolve = 0x4,
		af_no_acquire = 0x8,
		af_no_release = 0x10,
		
	};
	virtual void assign( value_ref left, value_ref right, builder & where, builder & store, unsigned flags = 0 ) = 0;

	enum release_flags
	{
		//release whatever is stored at the lvalue at the time of release (otherwise release current value)
		rf_cur_value = 0x1,
		//force inlning of the release, otherwise will call a common function
		rf_inline = 0x2,
		//the value to release is the pointed to value, not the immediate value
		rf_pointer = 0x4,
	};
	virtual void release( value_ref value, builder & where, block_flow flow, unsigned flags = 0) = 0;
	virtual void release_pointer( value_ref ptr, builder & where, block_flow flow, unsigned flags = 0) = 0;
	virtual boost::optional<value_ref> get_release_pointer( value_ref value, builder & where, unsigned flags =  0 ) = 0;

	enum initialize_flags {
		//a provided value intiailizes the referred value, not the shared ptr itself
		if_init_refer_value = 1<<0,
		//the value of the object is undefined (shared pointers are still created)
		if_undefined_value = 1<<1,
	};
	virtual value_ref initialize( value_ref ptr, shared_ptr<type const> init_type, 
		unsigned flags, builder & where ) = 0;

};

} //eon ir

#endif