#ifndef LEAF_IR_COMPILER_LEAF_HPP
#define LEAF_IR_COMPILER_LEAF_HPP

#include "compiler.hpp"

#include "../lang/fwd.hpp"

namespace ir
{

/**
	Leaf specific compiler (translates from Leaf code to Leaf-IR code)
*/
class compiler_leaf : public compiler
{
	friend class type_manager_leaf;
	
	shared_ptr<leaf::scope> globals;
	shared_ptr<type_manager_leaf> types;

	type_manager & get_types() override;
public:
	compiler_leaf( shared_ptr<leaf::scope> globals );

	shared_ptr<ir::module> compile_expr( leaf::expression const & expr );
	shared_ptr<ir::module> compile_block( leaf::statement_block const & bk,
		leaf::statement_block const & init );
	shared_ptr<ir::module> compile_module( leaf::module const & module );

protected:
	enum eval_flags
	{
		//we are binding to this value
		ef_bind = 0x1,
		//this value is being directly called as a function
		ef_calling = 0x2,
	};
	value_ref eval_single_expr( leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref_vector_t eval_list_expr( leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	void eval_list_expr( value_ref_vector_t & out, leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	void eval_funccall_expr( value_ref_vector_t & out, leaf::expr_funccall const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_subscript( leaf::expr_subscript const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_bind( leaf::expr_bind const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_field( leaf::expr_field const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_special_field( leaf::expr_special_field const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_transform( leaf::expr_transform const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_clone_func( leaf::expr_bind const & expr, builder & where );
	value_ref eval_binary( leaf::expr_binary const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_unary( leaf::expr_unary const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_conditional( leaf::expr_conditional const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_text( leaf::expr_text const & expr, builder & where );
	value_ref eval_switch( leaf::expr_switch const & expr, builder & where );
	value_ref eval_current( leaf::expr_current const & expr, builder & where );
	value_ref eval_temp( leaf::expr_temp const & expr, builder & where );
	value_ref eval_tag( leaf::expr_tag const & expr, builder & where );
	
	void build_expression_statement( leaf::expression_statement const & expr, builder & where );
	void build_statement( leaf::statement const & st, builder & where );
	void build_init_statement( leaf::init_statement const & vs, builder & where );
	void build_do_statement( leaf::do_statement const & on, builder &  where );
	void build_error_statement( leaf::error_statement const & es, builder & where );
	value_ref build_function( leaf::expr_funcdefn const & expr, builder & where );
	void build_capture_shared( leaf::expr_funcdefn const & expr, builder & where );
	void build_defer_statement( leaf::defer_statement const & ds, builder & where );
	void build_return_statement( leaf::return_statement const & ds, builder & where );
	void build_fail_statement( leaf::fail_statement const & fs, builder & where );
	void build_loop_flow_statement( leaf::loop_flow_statement const & ls, builder & where );

	void build_imperative_block( leaf::statement_block const & bk, builder & where );
	void build_declarative_block( leaf::statement_block const & bk, builder & where );
	void build_block_impl( leaf::statement_block const & bk, builder & where );
	
	void build_assign_statement( leaf::assign_statement const& as, builder &  where );

	void initialize_leaf( value_ref ptr, leaf::extr_type const & init_type, 
		shared_ptr<leaf::expression> value, unsigned flags, builder & where );

	value_ref obtain_symbol_value_ref( shared_ptr<leaf::declaration> sr );
};

} //eon ir
#endif