#ifndef LEAF_IR_FUNCTION_HPP
#define LEAF_IR_FUNCTION_HPP

#include "value.hpp"
#include "fwd.hpp"

#include <memory>
#include <vector>

namespace ir
{

class function : public std::enable_shared_from_this<function>
{
	enum private_ctor_t { private_ctor };
public:
	compiler_context & cctx;
	function( compiler_context & cctx, shared_ptr<value> use_value, private_ctor_t );

	static shared_ptr<function> create( compiler_context & cctx,
		shared_ptr<value> use_value )
	{ return make_shared<function>(cctx, use_value, private_ctor ); }
	
	virtual void validate( util::validation_context & vctx ) const;
	
	shared_ptr<value> defn_value;
	shared_ptr<block> entry_block; //never null
	
	//created on build_function if it contains refer closures
	boost::optional<value_ref> local_context;
	
	//these are the values as local to the function
	std::vector<value_defn> args;
	std::vector<value_defn> returns;

	//context_args are the values captured from the context_arg
	std::vector<value_defn> context_args;
	value_defn context_arg;

	std::vector<shared_ptr<value>> locals;
	
	unsigned anon_count;

	bool is_export;
	
	shared_ptr<value> make_temp( shared_ptr<type const> dt );
	std::string create_temp_name();
};

} //eon ir
#endif