#ifndef LEAF_IR_TYPE_MANAGER_HPP
#define LEAF_IR_TYPE_MANAGER_HPP

#include "fwd.hpp"

#include <set>
#include <map>

namespace ir
{

/**
	For efficiency the type_manager tries to merge all equivalent types. Higher level code
	should not assume this however as other things make "type" instances as well. In practice
	it was too troublesome to ensure each unique type has only one instance.
*/
class type_manager : public std::enable_shared_from_this<type_manager> {

	shared_ptr<type const> type_boolean, type_empty;
	std::map<int,shared_ptr<type const>> integer_cache, binary_cache;

protected:
	struct type_key {
		bool shared = false, optional = false, lvalue = false, value_ptr = false, array_ptr = false;
		intptr_t cref = 0;
		shared_ptr<type const> cached;
		
		bool operator<( type_key const & o ) const {
			if( cref != o.cref )
				return cref < o.cref;
			if( lvalue != o.lvalue )
				return lvalue < o.lvalue;
			if( shared != o.shared )
				return shared < o.shared;
			if( optional != o.optional )
				return optional < o.optional;
			if( value_ptr != o.value_ptr )
				return value_ptr < o.value_ptr;
			if( array_ptr != o.array_ptr )
				return array_ptr < o.array_ptr;
			return false;
		}

	};
	std::set<type_key> known_type_refs;

	type_manager();
public:
	
	shared_ptr<type const> get_boolean();
	shared_ptr<type const> get_integer( int bitsize );
	shared_ptr<type const> get_natural_integer();
	shared_ptr<type const> get_binary( int bitsize );
	shared_ptr<type const> get_binary_for_pointer();
	shared_ptr<type const> get_octet() {
		return get_binary(8);
	}
	shared_ptr<type const> get_empty();
	
	shared_ptr<type const> get_raw_pointer( shared_ptr<type const> t );
	shared_ptr<type const> get_deref( shared_ptr<type const> t );
	shared_ptr<type const> get_deref_shared( shared_ptr<type const> t );
	shared_ptr<type const> get_shared_pointer( shared_ptr<type const> t );
	shared_ptr<type const> get_raw_shared( shared_ptr<type const> t );
	shared_ptr<type const> get_array( shared_ptr<type const> t );
	shared_ptr<type const> remove_modifiers( shared_ptr<type const> t );
};

} //eon ir

#endif