#ifndef LEAF_IR_TYPE_MANAGER_LEAF_HPP
#define LEAF_IR_TYPE_MANAGER_LEAF_HPP

#include "type_manager.hpp"
#include "lang/fwd.hpp"
#include "util/ref_cache.hpp"

namespace ir
{

class type_manager_leaf : public type_manager
{
	shared_ptr<type const> get_fun( shared_ptr<leaf::intr_type const> fun );

	//use the intrusive pointer to track all fundamental types
	util::ref_cache<shared_ptr<type const>> known_fun_types;
	
	//track equivalent types as well, the higher level will have all sorts of anonymous types
	//which are actually the same
	struct leaf_type_key {
		shared_ptr<leaf::intr_type const> leaf_type;
		shared_ptr<type const> ir_type;
		
		bool operator<( leaf_type_key const & o ) const;
	};
	std::set<leaf_type_key> leaf_type_cache;

	compiler_leaf & comp;
	type_manager_leaf( compiler_leaf & comp );
public: 
	static shared_ptr<type_manager_leaf> create( compiler_leaf & comp ) {
		return shared_ptr<type_manager_leaf>(new type_manager_leaf(comp));
	}
	
	shared_ptr<type const> get( leaf::extr_type const & t );
	shared_ptr<type const> get( leaf::type_ref const & t );
};

} //eon ir

#endif