#ifndef LANG_CERR_HPP
#define LANG_CERR_HPP

#include "fwd.hpp"

namespace leaf
{

#define CERR_ENUMS_REST \
	CERR_ENUM(disallow_value_list) \
	CERR_ENUM(expect_extr_type) \
	CERR_ENUM(incomplete) \
	CERR_ENUM(invalid_array_cvt) \
	CERR_ENUM(invalid_context) \
	CERR_ENUM(invalid_expression) \
	CERR_ENUM(invalid_size) \
	CERR_ENUM(invalid_symbol_dot) \
	CERR_ENUM(invalid_symbol_type) \
	CERR_ENUM(invalid_type_ref) \
	CERR_ENUM(large_rational_integer) \
	CERR_ENUM(mismatch_array_ptr) \
	CERR_ENUM(mismatch_data_size) \
	CERR_ENUM(mismatch_defined) \
	CERR_ENUM(mismatch_field_name) \
	CERR_ENUM(mismatch_function_access) \
	CERR_ENUM(mismatch_function_throw) \
	CERR_ENUM(mismatch_fundamental) \
	CERR_ENUM(mismatch_lvalue) \
	CERR_ENUM(mismatch_optional) \
	CERR_ENUM(mismatch_params) \
	CERR_ENUM(mismatch_shared) \
	CERR_ENUM(mismatch_signed) \
	CERR_ENUM(mismatch_size) \
	CERR_ENUM(mismatch_type_ref) \
	CERR_ENUM(mismatch_value_ptr) \
	CERR_ENUM(missing_expression) \
	CERR_ENUM(missing_intrinsic) \
	CERR_ENUM(null_spec) \
	CERR_ENUM(precision_loss) \
	CERR_ENUM(range_loss) \
	CERR_ENUM(redundant_array_ptr) \
	CERR_ENUM(redundant_bitsize) \
	CERR_ENUM(redundant_cvt_optional) \
	CERR_ENUM(redundant_function_access) \
	CERR_ENUM(redundant_function_context) \
	CERR_ENUM(redundant_fundamental) \
	CERR_ENUM(redundant_literal) \
	CERR_ENUM(redundant_lossy) \
	CERR_ENUM(redundant_lvalue) \
	CERR_ENUM(redundant_member) \
	CERR_ENUM(redundant_optional) \
	CERR_ENUM(redundant_pack) \
	CERR_ENUM(redundant_parametric) \
	CERR_ENUM(redundant_selectable) \
	CERR_ENUM(redundant_shared) \
	CERR_ENUM(redundant_signed) \
	CERR_ENUM(redundant_spec) \
	CERR_ENUM(redundant_value_ptr) \
	CERR_ENUM(requires_function) \
	CERR_ENUM(requires_literal) \
	CERR_ENUM(requires_lvalue) \
	CERR_ENUM(requires_optional) \
	CERR_ENUM(requires_refer) \
	CERR_ENUM(requires_scalar) \
	CERR_ENUM(requires_shared) \
	CERR_ENUM(requires_tuple) \
	CERR_ENUM(requires_unqualified) \
	CERR_ENUM(requires_value) \
	CERR_ENUM(requires_value_list) \
	CERR_ENUM(requires_value_ptr) \
	CERR_ENUM(select_multiple) \
	CERR_ENUM(select_none) \
	CERR_ENUM(unbound_rational) \
	CERR_ENUM(unexpected_size) \
	CERR_ENUM(unify_ambiguous) \
	CERR_ENUM(unify_impossible) \
	CERR_ENUM(unknown_identifier) \
	CERR_ENUM(unknown_variant) \
	CERR_ENUM(unresolved_param) \
	CERR_ENUM(unsupported) 
	
#define CERR_ENUMS \
	CERR_ENUM(okay)\
	CERR_ENUMS_REST

enum class cerr
{
	okay = 0,
	#define CERR_ENUM(cerr) cerr,
	CERR_ENUMS_REST
	#undef CERR_ENUM
};

/**
	A common error result for the typing phase.
*/
struct cerr_base
{
	//which error was encoutnered, cerr_okay if none
	cerr error;
	std::string error_extra;
	std::string explain_error() const;
	
	bool okay() const 
	{ return error == cerr::okay; }
	/**
		Use this function instead of comparing directly, it will allow adding other logical
		classes of "unknown" errors. This allows out-of-order declaration processing
		in the typer.
	*/
	bool unknown_identifier() const
	{ return error == cerr::unknown_identifier; }
	
	cerr_base( cerr result = cerr::okay )
		: error( result )
	{ }
	cerr_base( cerr result, std::string const & msg )
		: error( result )
		, error_extra( msg )
	{ }
};

/**
	Mosts uses simply need to store one field additional to the base and then have a series
	of convenience functions.
*/
template<typename T>
struct cerr_impl : public cerr_base
{
	T result;
	
	/*implicit*/ cerr_impl( T const & result )
		: cerr_base( cerr::okay )
		, result( result )
	{ }
	/*implicit*/ cerr_impl( cerr err )
		: cerr_base( err )
	{ }
	cerr_impl( cerr err, std::string const & msg )
		: cerr_base( err, msg )
	{ }
	//explicit to prevent accidentla cross-type conversion
	explicit cerr_impl( cerr_base base )
		: cerr_base( base )
	{ }
	cerr_impl() { }
};

};
#endif