#ifndef LEAF_DO_STATEMENT_HPP
#define LEAF_DO_STATEMENT_HPP

#include "statement.hpp"

#include <vector>

namespace leaf {

struct expression;

/**
	The "do" statement executes the result of an expression. Unlike a typical function call this will
	allow special behaviour over just calling the function result:
		- a raw statement_block can be executed
		- no catch-all condition is required
*/
struct do_statement : public statement {
	do_statement() {
		loop = false;
	}
	
	virtual statement_form_t statement_form() const { 
		return sf_do_statement; 
	}
	virtual int get_flow() const;
	
	struct expr_list_item
	{
		shared_ptr<expression> cond; //if null then treat as always true (else clause)
		shared_ptr<statement_block> block;
	};
	typedef std::vector<expr_list_item> expr_list_t;
	expr_list_t expr_list;

	//loop on the items until the conditions are all false
	bool loop;
	
	STATEMENT_PRE_CLONE(do_statement);
	STATEMENT_SERIAL()
protected:
	void pre_copy( do_statement const & o );
	
	void write_impl( serial_out & so ) const;
	void read_impl( serial_in & so );
};

} //eon leaf

#endif