#ifndef LEAF_EXPRESSION_STATEMENT_HPP
#define LEAF_EXPRESSION_STATEMENT_HPP

#include "statement.hpp"

namespace leaf {

struct expression;

/**
	A statement which evalutes a single expression. This needs to be distinct from the
	expression itself since it has properties only a statement can have (like error handling)
*/
struct expression_statement : public statement {
	expression_statement() {
		ignore = false;
	}
	
	statement_form_t statement_form() const override { 
		return sf_expression_statement; 
	}
	int get_flow() const override {  
		return flow_sequence | flow_skippable; 
	}
	
	shared_ptr<expression> expr;
	//value of this expression is ignored
	bool ignore;
	
	void validate(util::validation_context & vctx) const override;

	STATEMENT_PRE_CLONE(expression_statement);
	STATEMENT_SERIAL()
protected:
	void pre_copy( expression_statement const & o);
	
	void write_impl( serial_out & so ) const;
	void read_impl( serial_in & si );
};

}

#endif