#ifndef LANG_EXPRESSION_TYPER_HPP
#define LANG_EXPRESSION_TYPER_HPP

//these are common includes used by most expression_typer functions
#include "lang/typer.hpp"
#include "lang/language.hpp"
#include "lang/conversion_applicator.hpp"
#include "lang/type_manager.hpp"

#include "lang/dump.hpp"

//CPP: There is unfortunately no way to use a function for this pattern
#define COND_PROC_EXPRESSION( expr, ctx, ret_value ) \
	STMT_CHECK( *ret_value, expr ); \
	expr = process_expression( expr, ctx ); \
	if( expr->get_extant() < extant_level::resolved ) { \
		return ret_value; \
	}
	
namespace leaf {

/**
	Focuses on typing expressions.
*/
namespace expression_typer {

	/**
		A composite result providing a new expression and whether processing
		should actually continue.
	*/
	struct result {
		shared_ptr<expression> expr;
		bool done;
		
		result( shared_ptr<expression> expr ) {
			this->expr = expr;
			done = true;
		}
		template<typename T>
		result( shared_ptr<T> expr ) {
			this->expr = expr;
			done = true;
		}
		
		operator bool() const {
			return done;
		}
		shared_ptr<expression> operator*() const {
			return expr;
		}
		
		result() {
			done = false;
		}
	};
	
	shared_ptr<expression> process_expression( shared_ptr<expression> expr, 
		context ctx );
	shared_ptr<expr_bind> bind_function( shared_ptr<expression> expr,
		context ctx );

	#define EXPRESSION_FORM(form) \
		shared_ptr<expression> process_##form( shared_ptr<expr_##form> expr, context ctx );
	EXPRESSION_FORMS
	#undef EXPRESSION_FORM

	result process_funccall_builtin( shared_ptr<expr_funccall> expr, context ctx );
	result process_funccall_type( shared_ptr<expr_funccall> expr, context ctx );
	result process_funccall_ctor( shared_ptr<expr_funccall> expr, context ctx );
	shared_ptr<expression> access_field_index( shared_ptr<expression> ex, 
		shared_ptr<expression> base, size_t index, context ctx );
		
	util::number extract_literal_number( expression const & expr );
	
	shared_ptr<expr_compound> access_literal_tuple( shared_ptr<expression> expr );
	
	shared_ptr<expr_funccall> create_default_ctor_call( util::source_provider const & sl, context ctx, type_ref t ); 
	shared_ptr<expression> resolve_scope_symbol_type( scope_symbol const & msym,
		shared_ptr<type_spec const> param, std::string symbol );
} //eon expression_typer

} //eon leaf

#endif