#ifndef INIT_STATEMENT_HPP
#define INIT_STATEMENT_HPP

#include "statement.hpp"

namespace leaf
{

struct expression;

/**
	Note that "code" can't be done simply by having a code sequence follow the init_statement. 
	The init_statmenet may also involve a deferred cleanup, and this cleanup will not be registered
	unless "code" completes (which is the desired effect).
*/
struct init_statement : public statement
{
	virtual statement_form_t statement_form() const {
		return sf_init_statement; 
	}
	
	shared_ptr<declaration> sref;
	//if null then use the default value
	shared_ptr<expression> value;
	//the code which initializes the variable (when a simple value is not enough)
	shared_ptr<statement> code;
	//initializes the value of a reference rather than the reference itself
	bool init_refer_value;
	//no default should be set, fundamental value is undefined
	bool undefined;
	//in normal flow no release should be generated, matched by a destructor, or is a return value
	//in error flow the release will be done
	bool no_release;
	
	init_statement() 
		: init_refer_value( false )
		, undefined( false )
		, no_release( false )
	{ }
	
	void validate(util::validation_context & vctx) const;
};

} //eon leaf

#endif