#ifndef LEAF_MODULE_HPP
#define LEAF_MODULE_HPP

#include "fwd.hpp"

#include <set>
#include <boost/optional.hpp>

namespace leaf {

/**
	Encapsulates all the high-level Leaf code that constitutes a module. It is primarily built by
	the typer.
*/
struct module {
	//the root of scopre/namespace tree
	shared_ptr<scope> root;
	//the name of the module (if a module)
	boost::optional<std::string> name;
	
	/** 
		The global scope for all declarations in this module. For an executable the global and
		root will be the same scope. For a module they will differ, since all the modules definitions
		go into global, but imports will be placed into root.
	*/
	shared_ptr<scope> global;
	
	/**
		The list of declarations that are to be exported from the module.
	*/
	std::vector<shared_ptr<declaration>> var_exports;
	std::vector<shared_ptr<intr_type>> type_exports;

	/**
		List of libraries that have been referenced by this module. This will need to be linked
		to execute the code.
	*/
	std::set<std::string> libraries;
	
	/**
		All static initializations should be placed into this block in an appropriate order.
	*/
	shared_ptr<statement_block> static_init_block;
	
	/**
		All the blocks that make up the module (excepting those specifically listed
		here, like static_init_block. The order of addition is preserved -- most relevant 
		for dynamic blocks.
	*/
	std::vector<shared_ptr<statement_block>> blocks;

	/**
		All function definitions in the module.
	*/
	std::set<shared_ptr<expr_funcdefn>> funcdefns;
	
	/**
		Check if all components are valid.
	*/
	void validate(util::validation_context & vctx) const;
};

} //eon leaf
#endif