#ifndef LANG_MODULE_LOADER_HPP
#define LANG_MODULE_LOADER_HPP

#include "../core_types.hpp"
#include "fwd.hpp"

namespace leaf {

/**
	Encapsulates the logic of locating and loading modules/libraries.
*/
class module_loader {
	std::string leaf_path;
	std::string dyn_ext, obj_ext, leaflib_ext, exe_ext	;
public:
	module_loader( std::string const & leaf_path );
	
	/**
		Return the appropriate link library for the given module name.
	*/
	std::string resolve_dynlink( std::string const & mod_name );
	
	/**
		Return the appropriate file for dynamic loading.
	*/
	std::string resolve_dynload( std::string const & mod_name );
	
	/**
		Resolve a static core link dependency. For building static
		executables.
	*/
	std::string resolve_static_core( std::string const & part );
	
	/**
		Resolve a dynamic core link dependency. For linking into
		shared objects and executables.
	*/
	std::string resolve_dyn_core( std::string const & part );
	
	/**
		Resolve an import name (get the Leaf module definition name) to a filename.
	*/
	std::string resolve_import( std::string const & part );
	
	std::string obj_name( std::string const & base );
	std::string dyn_name( std::string const & base );
	std::string exe_name( std::string const & base );
	std::string leaflib_name( std::string const & base );
};

}

#endif
