#ifndef STATEMENT_BLOCK_HPP
#define STATEMENT_BLOCK_HPP

#include "core_types.hpp"
#include "statement.hpp"
#include "fwd.hpp"

namespace leaf {

/**
	A block of statements with an associated scope.
*/
struct statement_block {
	statement_block( scope_style_t scope_style );
	statement_block( serial_in & si );
	~statement_block();
	
	shared_ptr<scope> symbols; //never null
	std::vector<shared_ptr<statement> > list;

	//a way to identify this block for debugging purpose (no semantic value)
	std::string reference_text;
	
	scope_style_t get_scope_style() const;
	
	/**
		Update the "flow" field from the list of blocks. Called from the type after the
		blocks have been typed.
	*/
	void update_flow();
	
	//TODO: merge flow reporting with statement somehow
	bool is_terminator() const { 
		return (get_flow() & ~flow_sequence); 
	}
	int get_flow() const { 
		return flow; 
	}
	
	//usable by compiler
	mutable intptr_t cref;
	
	void validate(util::validation_context & vctx) const;

	extant_level get_extant() const;
	bool is_complete() const {
		return get_extant() == extant_level::complete;
	}
	
	shared_ptr<statement_block> pre_clone() const;

	void write(serial_out & so);
	
	boost::optional<util::source_location> src_loc;
private:
	//all flow which leaves this block
	int flow; //POST-TYPE (via update_flow)
};

} //eon leaf

#endif