#ifndef LANG_STATEMENT_VISITOR_HPP
#define LANG_STATEMENT_VISITOR_HPP

#include "fwd.hpp"
#include "statement.hpp"

namespace leaf
{

struct expression_transformer;

/**
 	This is a transforming visitor of a statement. Return values form the apply methods will
 	replace the caller's reference.
 	
 	CPP: the "apply_" methods need to have full names and can't be overloads. The resolution
 	on shared pointers falls apart once this class is inherited in combination with expression_transformer.
*/
struct statement_transformer
{
	/**
	 	@param no_default_pass if false the default apply will simply return the input, if true
 			it will throw
	*/
	statement_transformer( bool no_default_pass = false );
	
	shared_ptr<statement> apply_statement( shared_ptr<statement> in, shared_ptr<scope> sc );
	void apply_statement_block( shared_ptr<statement_block> block );
protected:
	bool no_default_pass;
	
	//intercept statement functions do the basic handling and may be overridden (this will
	//block the default child traversal)
	#define STATEMENT_FORM(form) \
		virtual shared_ptr<statement> istmt_##form( shared_ptr<form##_statement> in, shared_ptr<scope> sc );
	STATEMENT_FORMS
	#undef STATEMENT_FORM
};

struct stat_expr_transformer : public statement_transformer
{
	stat_expr_transformer( expression_transformer & etrans);
	
protected:
	expression_transformer & etrans;
	
	shared_ptr<statement> istmt_init( shared_ptr<init_statement> in, shared_ptr<scope> sc );
	shared_ptr<statement> istmt_var( shared_ptr<var_statement> in, shared_ptr<scope> sc );
	shared_ptr<statement> istmt_type( shared_ptr<typedef_statement> in, shared_ptr<scope> sc );
	shared_ptr<statement> istmt_expression( shared_ptr<expression_statement> in, shared_ptr<scope> sc );
};

} //eon leaf
#endif