#ifndef LANG_TYPE_HPP
#define LANG_TYPE_HPP

#include "fwd.hpp"

#include "util/validation_context.hpp"

namespace leaf {
/**
	A common interface for all nodes in the type tree. Nothing is meant to work with items
	of `type_node`, only the implementing classes.
*/
struct type_node {
	/**
		Fixated types are those that have a concrete representation and can exist at runtime.
	*/
	bool is_fixated() const {
		util::validation_context vctx;
		return is_fixated_impl( vctx );
	}
	virtual bool is_fixated_impl( util::validation_context & vctx ) const = 0;

	extant_level get_extant() const {
		util::validation_context vctx;
		return get_extant_impl( vctx );
	}
	virtual extant_level get_extant_impl( util::validation_context & ) const = 0;
	
	// all parts of the tree are known and complete, without any parametrics
	bool is_complete() const {
		return get_extant() == extant_level::complete;
	}
	// all of the parts are resolved (their type won't change anymore)
	bool is_resolved() const {
		return get_extant() >= extant_level::resolved;
	}
	// the tree would be complete except for parametric components
	bool is_level_parametric() const {
		return get_extant() == extant_level::parametric;
	}
	
	/**
		Indictes whether any part of this type tree is parametric.
	*/
	bool contains_parametric() const {
		util::validation_context vctx;
		return contains_parametric_impl( vctx );
	}
	virtual bool contains_parametric_impl( util::validation_context & ) const = 0;

	void validate() const {
		util::validation_context vctx;
		validate( vctx );
	}
	virtual void validate( util::validation_context & vctx ) const = 0;
	
	virtual ~type_node() { 
	}
protected:
	type_node() = default;
	type_node( type_node const & ) = default;
};

} //eon leaf

#include "extr_type.hpp"
#include "type_ref.hpp"
#include "intr_type.hpp"

#include "extr_type.inl"

#endif