#ifndef LEAF_PARSER_NODE_CONVERTER_HPP
#define LEAF_PARSER_NODE_CONVERTER_HPP

#include "../core_types.hpp"
#include "../lang/fwd.hpp"
#include "fwd.hpp"

namespace leaf
{

struct node_converter
{
	typedef std::vector<shared_ptr<statement> > statement_vector_t;
	
	static shared_ptr<expression> expression_convert( node const & src );
	static shared_ptr<statement> convert( node const & src );
	static statement_vector_t convert_block( node const & src, scope_style_t );
	static shared_ptr<type_spec> convert( node_type_spec const & ts ) {
		return (node_converter()).convert_type(ts);
	}
	
	//bool value unused, just a placeholder for this tag
	typedef boost::error_info<struct tag_converter_error,bool> error_converter;
	
private:
	shared_ptr<defer_statement> convert_defer_statement( node const & src, bool error );
	shared_ptr<do_statement> convert_do_statement( node const & src );
	shared_ptr<expression_statement> convert_expression_statement( node const & src );
	shared_ptr<expression_statement> convert_ignore_expression_statement( node const & src );
	shared_ptr<for_statement> convert_for_statement( node const & src );
	shared_ptr<return_statement> convert_return_statement( node const & src );
	shared_ptr<statement> convert_break_statement( node const & src, bool cont );
	shared_ptr<statement> convert_fail_statement( node const & src, fail_level );
	shared_ptr<statement> convert_import_statement( node const & src );
	shared_ptr<statement> convert_on_statement( node const & src );
	shared_ptr<statement> convert_xon_statement( node const & src );
	shared_ptr<statement> convert_resume_statement( node const & src );
	shared_ptr<statement> convert_statement( node const & src);
	shared_ptr<var_statement> convert_var_statement( node const & src );
	shared_ptr<statement> convert_conditional_statement( node const & src );

	shared_ptr<statement_block> create_statement_block( node const & src, scope_style_t );
	void convert_block( statement_vector_t & block, node const & src, scope_style_t );
	
	enum assign_how_t {
		ah_value,
		ah_init_refer,
		ah_default,
	};
	shared_ptr<assign_statement> convert_assign_statement( node const & src, assign_how_t how );

	shared_ptr<expression> convert_expression( node const & src );
	shared_ptr<expression> convert_field_expression( node const & src );
	shared_ptr<expression> convert_select_expression( node const & src );
	shared_ptr<expression> convert_ifelse_expression( node const & src );
	shared_ptr<expression> convert_funcdefn_expression( node const & src );
	shared_ptr<expression> convert_cond_expression( node const & src );	
	shared_ptr<expression> convert_optional_or_expression( node const & src );
	shared_ptr<expression> convert_optional_cond_expression( node const & src );
	shared_ptr<expression> convert_type_expression( node const & src );
	shared_ptr<expression> convert_subscript_expression( node const & src );
	shared_ptr<expression> convert_tag_expression( node const & src );
	
	shared_ptr<typedef_statement> convert_typedef_statement( node const & src, bool alias );
	shared_ptr<typedef_statement> convert_class_statement( node const & src );
	
	void convert_sub( expr_with_sub & out, node const & src );
	
	struct sig_assign {
		size_t index;
		shared_ptr<expression> value;
	};
	typedef std::vector<sig_assign> sig_assigns_t;
	shared_ptr<type_spec> convert_to_type( shared_ptr<type_spec> type,
		shared_ptr<node> args,
		shared_ptr<node> returns, sig_assigns_t & arg_assigns );
	shared_ptr<type_spec> convert_to_type( shared_ptr<node> args,
		sig_assigns_t * assigns );
	
	enum convert_type_flags {
		ct_create_empty = 1<<0,
	};
	shared_ptr<type_spec> convert_type( node_type_spec const & nts, unsigned flags = 0 );
	void convert_type_spec( type_spec & ts);
};

}
#endif