#ifndef NODE_PARSER_HPP
#define NODE_PARSER_HPP

#include "node.hpp"
#include "source.hpp"

#include <vector>

namespace leaf
{

struct node_parser
{
	node_parser() {
		reset();
	}

	void reset()
	{
		op_stack.clear();
		out_stack.clear();
		is_value = false;
	}

	static shared_ptr<node> parse_single( source & src );
	static shared_ptr<node> parse_as_block( source & src );
	
	//bool value unused, just a placeholder for this tag
	typedef boost::error_info<struct tag_converter_error,bool> error_parser;

	typedef boost::error_info<struct tag_error_symbol,std::string> error_symbol;

	static shared_ptr<node> parse_value_list( source & src )
	{ return parse_compound( src, '(', ')', node::f_value_list ); }
private:
	enum parse_flags_t {
		line_ends_terminate = 1<<0,
		first_token = 1<<2,
		param_ends_terminate = 1 <<3,
	};
	shared_ptr<node> parse( source & src, unsigned flags );
	
	static shared_ptr<node> parse_compound( source & src, src_symbol_t start, src_symbol_t end,
		node::form_t ct );
	static shared_ptr<node> parse_tuple( source & src ) { 
		return parse_compound( src, '[', ']', node::f_tuple ); 
	}
	static shared_ptr<node> parse_block( source & src ) { 
		return parse_compound( src, '{', '}', node::f_block ); 
	}
	static shared_ptr<node> parse_param_list( source & src ) { 
		return parse_compound( src, sym_open_param, sym_close_param, node::f_param_list ); 
	}
		
	static std::string parse_string( source & src, src_symbol_t end );

	void parse_annotations( source & src, node::annotate_map_t & anno );
	
	struct op
	{
		enum form_t
		{
			f_binary,
			f_unary,
			f_funccall,
			f_ternary,
		} form;

		//the precedent of the operator
		unsigned precedence;
		//is it a right associative operator
		bool right_assoc;

		std::string text;

		util::source_location src_loc;
		
		op( util::source_location src_loc )
			: precedence( 0 )
			, right_assoc( false )
			, src_loc( src_loc )
		{ }
	};
	
	op binary_op( std::string const & text, util::source_location loc );
	op unary_op( std::string const & text, util::source_location loc );
	
	std::vector<shared_ptr<node> > out_stack;
	std::vector<op> op_stack;

	//is the last item a value
	bool is_value;

	void push_op( op const & o );
	void push_out( shared_ptr<node> const & ex );
	void push_value( shared_ptr<node> const & value );
	void pop_op();
	shared_ptr<node> collect_out(util::source_location loc);
	
	bool parse_single_token( source & src, unsigned flags );
	bool parse_single_string( src_symbol_t nc, source & src );
	bool parse_single_operator( std::string op, source & src );
	bool parse_single_special( std::string special, source & src );

	void parse_class( std::string style, source & src );
	
	shared_ptr<node> pop_out( );
};

} //eon leaf

#endif
