#ifndef LEAF_UTIL_DUMP_BASE_HPP
#define LEAF_UTIL_DUMP_BASE_HPP

#include "../core_types.hpp"
#include <iosfwd>
#include <vector>
#include <boost/optional.hpp>

namespace leaf_util
{

/**
	Provides common dumping operations.
*/
class dump_base
{
	struct indent_item 
	{
		std::string bit;
		boost::optional<std::string> first_bit;
		bool shown_first;
	};
	std::vector<indent_item> indent_bits;
	bool flat;
	std::ostringstream * backing;
protected:
	std::ostream & out;

	std::string indent_str();
	//a line-end which is a require space
	char const * endl()
	{
		return flat ? " " : "\n";
	}
	//a line-end which isn't a required space
	char const * endopt()
	{
		return flat ? "" : "\n";
	}
	
	void indent( char const * bit = "    ", char const * first_bit = nullptr );
	void unindent( char const * end_bit = nullptr );
	
	void error( std::string const & msg );
	
	bool is_flat() const
	{ return flat; }
public:
	dump_base( std::ostream & out );
	dump_base();
	~dump_base();
	
	/**
		Single-line output in a fixed format suitable for test comparisons.
	*/
	void set_flat();
	
	//if default ctor was used returns the generated string
	std::string str();

	void header( std::string const & msg );
	
	void line_break() {
		out << endl() << indent_str();
	}
};

} //eon leaf_util
#endif