#ifndef UTIL_OBJECT_HOLDER_HPP
#define UTIL_OBJECT_HOLDER_HPP

#include <vector>

namespace util
{

template<typename T>
void object_holder_dtor( void * obj ) {
	T * tobj = static_cast<T*>(obj);
	delete tobj;
}

/**
	Allows using raw pointers for objects as shared_ptr was becoming increasingly annoying, and
	leads to troublesome circular references.
	
	NOTE: This only works for plain types which don't access other such types in their destructor
	(so essentially this makes obejct_holder a garbage collector).
*/
class object_holder 
{
public:
	~object_holder();
	
	void clear();

	template<typename T, typename ... CP>
	T * make( CP&& ... ctor_args ) {
		T * obj = new T( std::forward<CP>(ctor_args)... );
		entry e;
		e.obj = static_cast<void*>(obj);
		e.dtor = &object_holder_dtor<T>;
		entries.push_back(e);
		return obj;
	}
	
private:
	typedef void (*dtor_func)( void * );
	struct entry {
		//the created object
		void * obj;
		//a cleanup function
		dtor_func dtor;
	};
	std::vector<entry> entries;
};

}//eon util
#endif