#ifndef UTIL_REF_CACHE_HPP
#define UTIL_REF_CACHE_HPP

#include "core_types.hpp"

#include <boost/optional.hpp>
#include <vector>

namespace util
{

/**
	A simple cache which makes use of intrusive intptr_t values.
	
	Requirements:
	- T must be copyable
	- "ref" is initialized to zero (as 0 is used at the not-set value)
*/
template<typename T>
class ref_cache
{
	std::vector<T> cache;
public:
	boost::optional<T> if_get( intptr_t ref ) {
		if( !ref ) {
			return boost::optional<T>();
		}
			
		return cache[ref-1];
	}
	
	void set( intptr_t & ref, T const & value ) {
		STATE_CHECK( !ref );
		cache.push_back( value );
		ref = cache.size();
	}
};

} //eon

#endif