#ifndef LEAF_UTIL_SOURCE_LOCATION_HPP
#define LEAF_UTIL_SOURCE_LOCATION_HPP

#include "../core_types.hpp"

namespace util
{

/**
	The base class to "parse/source" this contains only the essential bits needed
	to identify a location post-parsing.
*/
class source_base {
public:
	virtual size_t line_at( size_t loc ) const = 0;
	virtual size_t column_at( size_t loc, size_t tab_size = 4 ) const = 0;
	virtual std::string get_name() const = 0;
};

struct source_location {
	shared_ptr<const source_base> src;
	size_t loc;
		
	size_t line() const { 
		return src->line_at( loc ); 
	}
	size_t column(size_t tab_size = 4) const { 
		return src->column_at( loc, tab_size ); 
	}
	std::string name() const { 
		return src->get_name(); 
	}
	
	typedef boost::error_info<struct tag_error,source_location> error;
};

class source_provider {
public:
	/**
		If set indicates the source location which best represents the location
		of this node/expression/statement.
	*/
	boost::optional<source_location> src_loc;
};

} //eon util

#endif