#ifndef UTIL_UNICODE_HPP
#define UTIL_UNICODE_HPP

#include <cstdint>
#include "../core_types.hpp"	
#include <vector>

namespace unicode {

typedef uint32_t char_code;
typedef std::vector<uint32_t> codestr;

struct code_bit {
	uint32_t code;
	int count;
};

/**
	Extract the next character code.
*/
code_bit next8( uint8_t const * str, size_t max );

/**
	Length in code points of a utf-8 string.
*/
size_t len8( uint8_t const * str, size_t size );
inline size_t len8( std::string const & str ) {
	return len8( reinterpret_cast<uint8_t const*>(str.c_str()), str.size() );
}

/**
	Decode a utf-8 string.
	
	@param str utf-8 encoded string
	@return vector of unicode code point values
*/
codestr decode8( uint8_t const * str, size_t size );
inline codestr decode8( std::string const & str ) {
	return decode8( reinterpret_cast<uint8_t const*>(str.c_str()), str.size() );
}

/**
	Encode a utf-8 string
	
	@param str vector of code point values
	@return utf-8 encoded string
*/
std::string encode8( uint32_t const * str, size_t size );
inline std::string encode8( codestr const & str ) {
	return encode8( &str[0], str.size() );
}
inline std::string encode8( uint32_t cp ) {
	return encode8( &cp, 1 );
}

/*
	Serialization
*/
void write( codestr const & str, std::ostream & out );
void read( codestr & str, std::istream & in );

} //eon unicode

#endif