#!/usr/bin/python3
# coding=utf-8
"""
	As root:
		Python3:
		ln -s pygments_lexer.py /usr/local/lib/python3.5/dist-packages/pygments/lexers/leaf.py
		cd /usr/local/lib/python3.5/dist-packages/pygments/lexers/
		python _mapping.py
		
"""
from pygments.lexer import RegexLexer
from pygments.token import *

import re

class LeafLexer(RegexLexer):
	name = "Leaf"
	aliases = ['leaf']
	filenames = ['*.leaf','*.lfb','*.lfx',"*.qlz"]
	mimetypes = ['text/x-leaf']
	flags = re.DOTALL | re.MULTILINE
	
	tokens = {
		'root': [
			(r'\s+', Text),
			(r'//.*?$', Comment.Singleline),
			(r'/\*', Comment.Multiline, 'comment'),
			(r'[:=><\+\/\-\*\^\.\?@!#%\|$]+', Operator),
			(r'([\,\{\}\[\]\(\)\.]|｢|｣|⁑)', Punctuation ),
			(r'(alias|class|defn|defm|ignore|import|literal|multi|service|type|var|type|typedef|typeof)\b', Keyword.Declaration),
			(r'(in|lossy|new|pack|select|this|unpack)\b', Keyword),
			(r'(critical_fail|break|pre_fail|continue|defer_fail|defer|do|else|fail|for|loop|next|on|resume_fail|return|then|trap)\b',
				Keyword),
			(r'(false|true)', Keyword.Constant),
			(r'(or|and|and_not|nand|nor|not|trap|unopt|xor)', Operator.Word),
			(r'(abi_double|abi_float|abi_int|abi_long_double|abi_long|abi_long_long|abi_pack|abi_ptr|abi_short|array|'
				r'binary|boolean|byte_pack|char|float|high|integer|low|lvalue|member|no_throw|normal|octet|optional|'
				r'rational|raw|selectable|shared|string|type_infer)\b', Keyword.Type),
			(r'[a-zA-Z_][a-zA-Z0-9_]*', Name.Other),
			(r'0x([0-9a-fA-F_]+)(\.[0-9a-fA-F_]+)?(e[0-9]+)?', Number),
			(r'0o([0-7_]+)(\.[0-7_]+)?(e[0-9]+)?', Number),
			(r'0b([01_]+)(\.[01_]+)?(e[0-9]+)?', Number),
			(r'([0-9_]+)(\.[0-9_]+)?(e[0-9]+)?', Number),
			(r'"(\\\\|\\"|[^"])*"', String.Double),
			(r"'(\\\\|\\'|[^'])*'", String.Single),
		],
		'comment': [
			(r'[^*/]', Comment.Multiline),
			(r'/\*', Comment.Multiline, '#push'),
			(r'\*/', Comment.Multiline, '#pop'),
			(r'[*/]', Comment.Multiline),
		],
	}

__all__ = [ 'LeafLexer' ]
