import re, os
from pprint import pprint

# Process the typical size notations used in the various sys files
def parse_size( text ):
	result = re.match( '^(\d+)(|K|M|G)$', text )
	if result == None:
		raise StandardError( "Unknown size format {0}".format( text ) )
		
	value = int(result.group(1))

	# Convert scale
	size = result.group(2)
	if size == 'K':
		value *= 1024
	elif size == 'M':
		value *= 1024 * 1024
	elif size == 'G':
		value *= 1024 * 1024 * 1024
	elif size == '':
		None
	else:
		raise StandardError( "Unknown size type {0}".format( size ) )
	
	return value

# Loads a size from a file
def load_size( name ):
	with open( name, 'r' ) as f:
		data = f.read()
		return parse_size( data )

# Load stats for a particular CPU
def load_cpu_stats( path ):
	stats = {}

	stats['cache_line'] = load_size( os.path.join( path,  'cache/index0/coherency_line_size' ) )
	stats['id'] = load_size( os.path.join( path, 'topology/core_id' ) )

	# There's all sorts of more stuff, but I don't need any of it now
	#stats['cache_l1_size'] = load_size( os.path.join( path,  'cache/index0/size'  ) )

	return stats

# Load stats for all CPUs listed
def load_cpu_stats_all():
	stats = []

	cpu_path = '/sys/devices/system/cpu/'
	for name in os.listdir( cpu_path ):
		result = re.match( '^cpu\d+$', name )
		if result == None:
			continue

		stats.append( load_cpu_stats( os.path.join( cpu_path, name ) ) )

	return stats

#pprint(load_cpu_stats_all(), indent=4, width=1)

# Loads the values which must be known at-compile time, or are helpful to know at
# compile time
def load_static_essentials():
	stats = {}
	all = load_cpu_stats_all()
	if len( all ) == 0:
		raise StandardError( 'Unable to load processor information' )
		
	stats['NUM_PROCESSORS'] = len( all )
	stats['CACHE_LINE_SIZE'] = all[0]['cache_line']

	return stats
