# Common code used by the test drivers
source build/env.sh

function error() {
	echo "$*"
	exit 1
}

# Set LOG_FILE for where log is written
# Set TEST_ITEMS for which tests to run
# Create function run_test which takes a TEST_ITEM as parameter and runs the test
function run_all() {
	echo $(date) > $LOG_FILE

	PASS=0
	FAIL=0
	SKIP=0

	for item in $TEST_ITEMS; do
		echo "" >> $LOG_FILE
		echo "[$item]" >> $LOG_FILE
		run_test "$item"
		
		pass=$?
		if [ $pass == 0 ]; then
			echo -e $item "\e[0;32mPASS\e[m"
			PASS=$(($PASS+1))
		elif [ $pass == 2 ]; then
			SKIP=$(($SKIP+1))
			echo -e $item "\e[0;33mSKIP\e[m"
		else
			echo -e $item "\e[0;31mFAIL\e[m"
			FAIL=$(($FAIL+1))
		fi
	done

	echo -en "$PASS of $(($PASS+$FAIL)) Passed"
	if [ $SKIP != 0 ]; then
		echo -en " (\e[0;33m$SKIP Skipped\e[m)"
	fi
	if [ $FAIL != 0 ]; then
		echo -e " (\e[0;31m$FAIL Failed\e[m)"
		exit 1
	fi
	echo ""
	return 0
}

O_LEAF_OPTS="--distinct-exception-flow"
O_NO_OPTS="--reduce-branch=0 --reduce-flow=0"
