#include "lang/dump.hpp"
#include "lang/serial.hpp"
#include "lang/declaration.hpp"
#include "lang/scope.hpp"

#include <boost/program_options.hpp>
namespace po = boost::program_options;
#include <boost/exception/diagnostic_information.hpp>

#include <vector>
#include <fstream>
#include <iostream>

void load_file( std::string const & name );

int main( int argc, char ** argv ) {

	std::vector<std::string> obj_files;
	
	po::options_description desc( "Options" );
	desc.add_options()
		( "help", "show help" )
		( "obj-file", po::value(&obj_files), "Leaf object files" )
		;
	po::positional_options_description p;
	p.add( "obj-file", -1 );
	
	po::variables_map vm;
	po::store( po::command_line_parser(argc,argv).options(desc).positional(p).run(), vm );
	po::notify( vm );
	
	if( vm.count( "help" ) ) {
		std::cerr << desc;
		return 0;
	}
	
	if( obj_files.empty() ) {
		std::cerr << "requires object file" << std::endl;
		return 1;
	}

	try {
		for( auto & of : obj_files ) {
			load_file( of );
		}
		return 0;
	} catch(...) {
		std::cerr << boost::current_exception_diagnostic_information();
		return 1;
	}

}

struct dump_serial_resolver : public leaf::serial_resolver {
	virtual boost::optional<std::string> get_ref_name( leaf::intr_type const & /*t*/ ) {
		STATE_FAIL( "unexpected" );
	}
	
	virtual std::shared_ptr<leaf::intr_type const> get_intr_type( std::string const & ref_name ) {
		//create a dummy placeholder
		auto unk = make_shared<leaf::intr_type_tuple>();
		unk->set_distinct_name( ref_name );
		return unk;
	}
};

void load_file( std::string const & name ) {
	std::ifstream in( name, std::ios::binary );
	dump_serial_resolver dsr;
	leaf::serial_in sin( in, dsr );
	leaf::dump d( std::cout );
	d.set_validate(false);
	d.set_omit_value(true);
	
	auto root = leaf::scope::create( leaf::scope_style_t::declarative );
	auto module = sin.get_key_root_scope();
	STATE_CHECK( module );
	root->set_base( root, leaf::scope::bt_namespace );
	root->set_name( "module" );
	
	for( auto & sitem : sin.get_all_items() ) {
		switch( sitem.type ) {
			case leaf::serial_item::t_invalid:
			case leaf::serial_item::t_end_marker:
			case leaf::serial_item::t_expression:
			case leaf::serial_item::t_scope:
			case leaf::serial_item::t_key_marker:
			case leaf::serial_item::t_type_spec:
			case leaf::serial_item::t_statement:
				break;
				
			case leaf::serial_item::t_type_ref:
			case leaf::serial_item::t_extr_type:
				//these aren't actually of direct interest
				break;
				
			case leaf::serial_item::t_intr_type:
				if( sitem.v_intr_type->is<leaf::intr_type_custom>() 
					|| sitem.v_intr_type->style == leaf::intr_type::s_distinct ) {
					d.write( sitem.v_intr_type );
					d.line_break();
				}
				break;
				
			case leaf::serial_item::t_declaration:
				if( sitem.v_declaration->var_scope == module  ) {
					d.write_declaration( *sitem.v_declaration );
					d.line_break();
				}
				break;
		}
	}
}
