#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
	Run a standalone Leaf program
"""
import argparse, os, subprocess, sys
import pyleaf
from shelljob import job, fs, proc

parser = argparse.ArgumentParser( description = 'Run leaf executable program' )
parser.add_argument( '--no-libs', action='store_const', const=True )
parser.add_argument( '--build', action='store_const', const=True )
parser.add_argument( '--dump-ir', action='store_const', const=True )
parser.add_argument( '--dump-llvm', action='store_const', const=True )
parser.add_argument( '--dump-post-type', action='store_const', const=True )
parser.add_argument( '--llvm-opt', action='store_const', const=True )
parser.add_argument( 'path', metavar='exe_dir', nargs = 1)
parser.add_argument( 'args', metavar='args', nargs='*')

args = parser.parse_args()
path = args.path[0].rstrip( '/' )
exe_name = os.path.basename( path )

# PYTHON: issues with writing bytes to stdout
bstdout = os.fdopen(sys.stdout.fileno(), 'wb')

class ExeMonitor(pyleaf.JobMonitor):
	
	def job_output(self, jobj, line):
		super(ExeMonitor,self).job_output(jobj,line)
		bstdout.write(line)

if not args.no_libs:
	monitor = ExeMonitor()
	leaf = pyleaf.LeafJob()
	leaf.build_libs( 'share/exe_tests/_libs', monitor )
	if not monitor.okay:
		print("Not okay")
		sys.exit(1)

try:
	files = fs.find( path, include_dirs = False, name_regex = '.*\\.leaf$' )
	exe_path = '/tmp/' + exe_name
	exe = [ 'build/leaf' ]
	if args.dump_ir:
		exe += [ "--dump-ir" ]
	if args.dump_llvm:
		exe += [ "--dump-llvm" ]
	if args.dump_post_type:
		exe += [ "--dump-post-type" ]
	if args.llvm_opt:
		exe += [ "--llvm-opt" ]
	exe += [ '--exe', exe_path ] 
	exe += list(files)
	print( " ".join(exe) )
	subprocess.check_call( exe )

	if not args.build:
		run = [ exe_path ] + args.args
		print( " ".join(run) )
		subprocess.check_call( run )
except:
	sys.exit(1)