#include "ir/block.hpp"
#include "ir/function.hpp"
#include "ir/instruction.hpp"

namespace ir {

shared_ptr<value> block::make_temp( shared_ptr<type const> dt ) { 
	return get_owner_function().make_temp( dt ); 
}

void block::validate( util::validation_context & vctx) const {
	STATE_CHECK( parent || owner_function );
	
	for( size_t i=0; i < items.size(); ++i ) {
		STATE_CHECK( items[i].an_instruction || items[i].a_block );
		if( items[i].an_instruction )
			items[i].an_instruction->validate(vctx);
			
		if( items[i].a_block )
			items[i].a_block->validate(vctx);
	}
}

void block::append( block & o ) {
	items.insert( items.end(), o.items.begin(), o.items.end() );
}

function & block::get_owner_function() const {
	if (owner_function) {	
		return *owner_function;
	}
	if (parent) {
		return parent->get_owner_function();
	} 
	STATE_FAIL( "corrupt-block" );
}

boost::optional<value_ref> block::find_error_info() const {
	if( error_info ) {
		return error_info;
	}
	if( parent ) {
		return parent->find_error_info();
	}
	return boost::optional<value_ref>();
}

} //eon ir