#include "ir/function.hpp"
#include "ir/block.hpp"
#include "util/validation_context.hpp"

#include <set>

namespace ir
{

function::function( compiler_context & cctx, shared_ptr<value> use_value, private_ctor_t )
	: cctx( cctx )
	, anon_count( 0 )
{
	is_export = false;
	
	if( !use_value ) {
		defn_value = make_shared<value>(value::f_global);
	} else {
		STATE_CHECK( !use_value->direct_type );
		defn_value = use_value;
		defn_value->form = value::f_global;
	}
	
	entry_block = block::create_root(this);
}

shared_ptr<value> function::make_temp( shared_ptr<type const> dt )
{
	auto v = make_shared<value>();
	v->form = value::f_temp;
	v->name = create_temp_name();
	v->direct_type = dt;
	return v;
}

std::string function::create_temp_name()
{
	anon_count++;
	char buf[128];
	sprintf( buf, "%%%d", anon_count );
	return buf;
}

void function::validate( util::validation_context & vctx) const
{
	if( vctx.cover(this) ) {
		return;
	}
		
	STATE_CHECK( defn_value );
	defn_value->validate(vctx);
	
	for( auto & p : args )
		p.validate(vctx);
	for( auto & r : returns )
		r.validate(vctx);
	for( auto & c : context_args )
		c.validate(vctx);
		
	for( auto & l : locals )
		l->validate(vctx);
		
	STATE_CHECK( entry_block );
	entry_block->validate(vctx);
}

} //eon ir