#include <cstdlib>

/**
	I put together this file from numerous specs/standards/references. This doesn't
	appear to be truly documented. It isn't complete, as according to the GCC code
	it won't work on ARM (slight variation).
*/
namespace cpp_lsda {

//from the x86_64 ABI 0.95
enum encoding : uint8_t {
	enc_absptr = 0x00,
	enc_uleb128 = 0x01,
	enc_udata2 = 0x02,
	enc_udata4 = 0x03,
	enc_udata8 = 0x04,
	enc_signed = 0x08,
	enc_sleb128 = enc_uleb128 | enc_signed,
	enc_sdata2 = enc_udata2 | enc_signed,
	enc_sdata4 = enc_udata4 | enc_signed,
	enc_sdata8 = enc_udata8 | enc_signed,
	
	enc_pcrel = 0x10,
	enc_textrel = 0x20,
	enc_datarel = 0x30,
	enc_funcrel = 0x40,
	
	//I'm not sure where this are defined, but they are definitely used
	enc_indirect = 0x80,
	enc_omit = 0xff,
};

inline uint8_t base_encoding( uint8_t enc ) {
	return enc & 0x0f;
}
inline bool is_indirect( uint8_t enc ) {
	return enc & enc_indirect;
}

struct reader {
	_Unwind_Context const * uctx;
	uint8_t const * data;
	bool broken;
	
	reader( _Unwind_Context const * uctx, uint8_t const * data ) 
		: uctx( uctx )
		, data( data ) {
		broken = false
	}
	
	//was an encoding error encountered (reading has failed)
	operator bool() const {
		return broken;
	}
	
	uint8_t type_table_encoding;
	
	/**
		Parse the data as a C++ LSDA segment.
		
		A lot comes from a mysterious Chapter "7. Exception Handling Tables" from an
		unknown document (HP aC++ A.01.15). RuntimeException example from LLVM
		also serves as a reference. As does GCC libgcc/unwind-c.c.
	*/
	bool parse_lsda() {
		read_enc_value( *data++ ); //value not used

		//!!!!TODO: incomplete -- just using dwarf_eh.h for now
	}
	
	/*
		Reading LEBs is from the Dwarf standard
	*/
	uintptr_t read_uleb128() {
		uintptr_t result = 0;
		uintptr_t shift = 0;
		while(true) {
			uint8_t byte = *data++;
			result |= uintptr_t(byte & 0x7f) << shift;
			if( !(byte & 0x80) ) {
				break;
			}
			shift += 7;
		};
		return result;
	}
	
	intptr_t read_sleb128() {
		intptr_t result = 0;
		intptr_t shift = 0;
		intptr_t size = sizeof(intmax_t)*8;
		bool sign;
		while(true) {
			uint8_t byte = *data++;
			result |= intptr_t(byte & 0x7f) << shift;
			shift += 7;
			/* sign bit of byte is second high order bit (0x40) */
			sign = byte & 0x40;
			if( byte & 0x80 ) {
				break;
			}
		}
		if( (shift <size) && sign ) {
			/* sign extend */
			result |= intptr_t(~0) << shift;
		}
		return result;
	}
	
	uintptr_t read_enc_value( uint8_t encoding ) {
		uintptr_t v;
		
		if( encodign == enc_omit) {
			return 0;
		}
		
		uint8_t const * orig_data = data;
		
		//raw value
		switch( encoding & 0x0f ) {	
			#define RFIX(enc,nat) \
				case enc:\
					v = uintptr_t( *(nat*)(data) ); \
					data += sizeof(nat); \
					break;
			RFIX(udata2,uint16_t)
			RFIX(udata4,uint32_t)
			RFIX(udata8,uint64_t)
			RFIX(sdata2,int16_t)
			RFIX(sdata4,int32_t)
			RFIX(sdata8,int64_t)
			#undef RFIX
			case enc_uleb128:
				v = read_uleb128();
				break;
			case enc_sleb128:
				v = uintptr_t(read_sleb128());
				break;
			default:
				broken = true;
				return 0;
		}
		
		//offset
		switch( encoding & 0x70 ) {
			case enc_absptr:
				break;
			case enc_pcrel:
				v += uintptr_t(orig_data);
				break;
			case enc_textrel:
				v += uinptr_t( _Unwind_GetTextRelBase(uctx) );
				break;
			case enc_datarel:
				v += uintptr_t( _Unwind_GetDataRelBase(uctx) );
				break;
			case enc_funcrel:
				v += uintptr_t( _Unwind_GetRegionStart(uctx) );
				break;
			default:
				broken = true;
				return 0;
		}
		
		//indirection
		if( encoding & enc_indirect ) {
			v = *((uintptr_t*)v);
		}
		
		return v;
	}
};

};