#include "ir/language.hpp"

namespace ir
{

module::module( std::string const & name, compiler_context & cctx )
	: cctx( cctx )
	, name( name ) {
}

shared_ptr<function> module::create_function( std::string const & name, 
	shared_ptr<value> use_value )
{
	function_map_t::iterator it = function_map.find( name );
	PRE_CHECK( it == function_map.end(), name );
	
	auto f = function::create(cctx,use_value);
	f->defn_value->name = name;
	
	function_map[name] = f;
	return f;
}

shared_ptr<function> module::if_get_function( std::string const & name ) const {
	auto it = function_map.find( name );
	if( it == function_map.end() )
		return shared_ptr<function>();
		
	return it->second;
}

void module::validate( util::validation_context & vctx) const {
	STATE_CHECK( name.size() );
	for( auto & f : function_map )
		f.second->validate(vctx);
	for( auto & g : global_variable_map )
		g.second->validate(vctx);
}

shared_ptr<value> module::if_get_global_variable( std::string const & name ) {
	auto git = global_variable_map.find( name );
	if( git == global_variable_map.end() )
		return shared_ptr<value>();
		
	return git->second;
}

void module::register_global( shared_ptr<value> g )
{
	PRE_CHECK( global_variable_map.find( g->name ) == global_variable_map.end(), 
		std::string( "duplicate-global:" + g->name ) );
	global_variable_map[g->name] = g;
}

} //eon ir