#include "ir/trace_functions.hpp"
#include "ir/builder.hpp"
#include "ir/type_manager.hpp"

namespace ir
{

void main_trace_function( compiler_context & cc, shared_ptr<ir::function> call_f )
{
	STATE_CHECK( call_f->defn_value->direct_type->args.size() == 0 );
	STATE_CHECK( call_f->defn_value->direct_type->sub.size() == 1 );
	auto ret_type = call_f->defn_value->direct_type->sub[0]->clone( false );
	
	module_builder mb(cc);
	auto fb = mb.create_function( "_entry", true );
	//no args/returns
	
	builder bb(fb);
	//call "f" to get return value
	auto fc = bb.add_funccall_inst( value_ref::as_value(call_f->defn_value) );
	auto ret_value = fc.define_return( ret_type );

	gen_trace( cc, bb, ret_value );
}

void gen_trace( compiler_context & cc, builder & where, value_ref trace_value )
{
	auto trace_type = trace_value.result_type();
	switch( trace_type->fundamental )
	{
		case type::f_integer:
		{
			trace_type = cc.get_types().get_natural_integer();
			trace_value = where.add_promote_loss( false, trace_value, trace_type, 
				promote_inst::w_integer, promote_inst::w_integer );
			where.call_builtin_unary( "trace_integer", trace_value );
			break;
		}
		
		case type::f_binary:
		{
			trace_type = cc.get_types().get_binary(platform::natural_integer_size);
			trace_value = where.add_promote_loss( false, trace_value, trace_type, 
				promote_inst::w_binary, promote_inst::w_binary );
			where.call_builtin_unary( "trace_binary", trace_value );
			break;
		}
		
		case type::f_float:
		{
			auto nf = platform::if_get_float( trace_type->trivial_bitsize );
			STATE_CHECK( nf );
			std::string name;
			if( nf->precision == -1 )
				name = "trace_float_low";
			else if( nf->precision == 1 )
				name = "trace_float_high";
			else if( nf->precision == 0)
				name = "trace_float";
			else {
				STATE_FAIL();
			}
			
			where.call_builtin_unary( name, trace_value );
			break;
		}
		
		case type::f_boolean:
		{
			where.call_builtin_unary( "trace_boolean", trace_value );
			break;
		}
		
		case type::f_tuple:
		case type::f_optional:
		{
			gen_write_char( cc, '[', where );
			
			for( size_t i=0; i < trace_type->sub.size(); ++i )
			{
				if( i )
					gen_write_char( cc, ',', where );
				gen_write_char( cc, ' ', where );

				gen_trace( cc, where, where.access_post_field(trace_value, i) );
			}
			
			gen_write_char( cc, ' ', where );
			gen_write_char( cc, ']', where );
			break;
		}
		
		default:
			STATE_CHECK(false);
	}
}

void gen_write_char( compiler_context & cc, char c, builder & where )
{
	auto ci = where.const_integer_normal( int(c) );
	
	auto t = make_shared<type>( cc.get_types().shared_from_this(), type::a_raw );
	t->args.push_back( ci.origin->direct_type );
	auto v = where.define_value( value::f_import_code, t, "trace_char" );
	
	auto fc = where.add_funccall_inst( v );
	fc.add_arg( ci );
}


} //eon ir