#include "ir/value.hpp"
#include "ir/type.hpp"
#include "ir/type_manager.hpp"

namespace ir {

bool value::is_any_storage() const { 
	return is_direct_storage() || direct_type->fundamental == type::f_raw_pointer; 
}

void value::validate( util::validation_context & vctx) const {
	STATE_CHECK( form != form_t(0) );
	STATE_CHECK( direct_type );
	direct_type->validate(vctx);
	
	switch( form )
	{
		case f_import_code:
		case f_import_data:
			STATE_CHECK(name.size());
			break;
		case f_const_integer:
			STATE_CHECK(direct_type->is_integer());
			break;
		case f_const_binary:
			STATE_CHECK(direct_type->is_binary());
			break;
		case f_const_float:
			STATE_CHECK(direct_type->is_float());
			break;
		case f_const_boolean:
			STATE_CHECK(direct_type->is_boolean());
			break;
		case f_temp:
		case f_undefined:
		case f_return_ref:
		case f_arg_ref:
		case f_capture_ref:
		case f_global:
		case f_local:
		case f_nullptr:
		case f_raw_binary:
			//nothing speical to check
			break;
	}
}

void value_defn::validate( util::validation_context & vctx) const
{
	STATE_CHECK( origin );
	origin->validate(vctx);
}

///////////////////////////////////////////////////////////
// value_ref
void value_ref::validate( util::validation_context & vctx) const {
	STATE_CHECK( origin );
	STATE_CHECK( how != how_t(0) );
	origin->validate(vctx);
	
	auto check_type = origin->direct_type;
	
	if( pre_field ) {
		if( origin->direct_type->is_raw_pointer() ) {
			auto under = origin->direct_type->sub[0];
			STATE_CHECK( *pre_field < under->sub.size() );
			return;
		} else {
			STATE_CHECK( *pre_field < check_type->sub.size() );
			check_type = check_type->sub[*pre_field];
		}
	}
	
	if( how == h_deref ) {
		STATE_CHECK( check_type->fundamental == type::f_raw_pointer 
			|| check_type->fundamental == type::f_shared_pointer );
	}
}

shared_ptr<type const> value_ref::result_type() const {
	STATE_CHECK(origin);
	STATE_CHECK(origin->direct_type);
	shared_ptr<type const> res = origin->direct_type;
	
	if( pre_field ) {
		if( origin->direct_type->is_raw_pointer() ) {
			auto under = origin->direct_type->sub[0];
			STATE_CHECK( *pre_field < under->sub.size() );
			res = res->get_types().get_raw_pointer( under->sub[*pre_field] );
		} else {
			STATE_CHECK( *pre_field < res->sub.size() );
			res = res->sub.at( *pre_field );
		}
	}
		
	switch( how ) {
		case h_value:
			break;
		case h_raw_pointer:
			res = res->get_types().get_raw_pointer( res );
			break;
		case h_deref:
			res = res->get_types().get_deref( res );
			break;
		case h_deref_shared:
			res = res->get_types().get_deref_shared( res );
			break;
		case h_raw_shared:
			res = res->get_types().get_raw_shared( res );
			break;
		case h_raw_array:
			//STATE_CHECK( res->fundamental == type::f_array && res->sub.size() == 1 );
			res = res->get_types().get_raw_pointer( res->sub[0] );
			break;
	}
	
	return res;
}

} //eon ir