#include "lang/do_statement.hpp"
#include "lang/statement_block.hpp"
#include "lang/expression.hpp"
#include "lang/serial.hpp"

namespace leaf {

int do_statement::get_flow() const {
	int combine = flow_sequence | flow_skippable;
	for( auto & eli : expr_list ) {
		combine |= eli.block->get_flow();
	}
	
	//loops end next/break flow
	if( loop )
		combine &= ~(flow_loop_break | flow_loop_next);
		
	return combine;
}

void do_statement::pre_copy( do_statement const & o ) {
	loop = o.loop;
	expr_list = o.expr_list;
	for( auto & e : expr_list ) {
		if( e.cond ) {
			e.cond = e.cond->pre_clone();
		}
		e.block = e.block->pre_clone();
	}
}

void do_statement::write_impl( serial_out & so ) const {
	so.write(loop);
	so.write( expr_list, [&]( expr_list_item const & a ) {
		so.write_ref( a.cond );
		a.block->write( so );
	});
}

void do_statement::read_impl( serial_in & si ) {
	si.read(loop);
	si.read( expr_list, [&]( expr_list_item & a ) {
		a.cond = si.read_expression_ref();
		a.block = make_shared<statement_block>( si );
	});
}

} //eon leaf