void dump::dump_expression( expression const * ex, unsigned flags ) {
	if( flags & ef_isolated ) {
		validate( *ex );
	}
		
	CHECK_NULL(ex, "expression");
	bool omit_type = false;
	bool list_close = false;

	//TODO: problem with certain var statements not showing their init
	/*if( ex->bound_decl ) {
		out << "“" << ex->bound_decl->symbol << "”";
		if( !(flags & ef_expand_name) ) {
			return;
		}
		out << " ";
	}*/
	
	//force value-list notation if type is included to prevent ambiguity
	if( (show_all_types && (ex->type.is_defined()) &&
		(ex->form != expression::f_compound && ex->form != expression::f_value_list
		&& ex->form != expression::f_funcdefn)
		&& !(flags & ef_isolated) ) || SHOW_COMPLETENESS )
	{
		out << "( ";
		list_close = true;
	}
	
	switch( ex->form )
	{
		case expression::f_symbol: {
			auto & es = static_cast<expr_symbol const&>(*ex);
			bool force_type = false;
			if( es.sref ) {
				switch( es.sref->form ) {
					case declaration::f_arg: out << "args."; break;
					case declaration::f_return: out << "returns."; break;
					case declaration::f_capture:  {
						out << "captures"; 
						auto level = es.sref->get_capture_level();
						if( level > 0 ) {
							out << "^" << level;
						}
						out << ".";
						break;
					}
					case declaration::f_single: /*force_type = true;*/ break;
					case declaration::f_multi: break;
				}
			}
			if( force_type && ex->type.is_defined() ) {
				//for multi showing the type is helpful
				omit_type = true;
				out << "( ";
				out << es.symbol;
				out << " : ";
				dump_type_ref( ex->type );
				out << " )";
			} else {
				out << es.symbol;
				if( es.param ) {
					out << unicode::encode8(sym_open_param);
					dump_type_spec( es.param.get() );
					out << unicode::encode8(sym_close_param);
				}
			}
			break;
		}
		
		case expression::f_literal_number: {
			auto & ecn = static_cast<expr_literal_number const&>(*ex);
			switch( ecn.src_form ) {
				case expr_literal_number::sf_rational: out << ecn.value.to_string(true); break;
				case expr_literal_number::sf_integer: out << ecn.integer_value(); break;
				case expr_literal_number::sf_binary: out << ecn.integer_value(); break; //TODO: hmm...
				case expr_literal_number::sf_float: out << ecn.float_value(); break;
				case expr_literal_number::sf_boolean: out << (ecn.boolean_value() ? "true" : "false"); break;
			}
			break;
		}

		case expression::f_text:
		{
			auto & es = static_cast<expr_text const&>(*ex);
			switch( es.what ) {
				case expr_text::w_string:
					out << '\"' << unicode::encode8(es.text) << '\"'; //TODO: proper format
					break;
				case expr_text::w_char:
					out << "\'" << unicode::encode8(es.text) << "\'";
					break;
			}
			break;
		}

		case expression::f_funccall:
		{
			auto & ef = static_cast<expr_funccall const&>(*ex);
			if( ef.context ) {
				dump_expression( ef.context.get() );
				out << "~";
			}
			dump_expression( ef.func.get() );
			out << " <- ";
			dump_expression( ef.args.get() );
			break;
		}

		case expression::f_compound:
			dump_tuple( static_cast<expr_compound const*>(ex), '[', ']', ',' );
			break;

		case expression::f_value_list:
			dump_tuple( static_cast<expr_value_list const*>(ex), '(', ')', ',' );
			break;
			
		case expression::f_funcdefn:
			dump_funcdefn( static_cast<expr_funcdefn const&>(*ex) );
			break;
			
		case expression::f_switch:
		{
			auto & es = static_cast<expr_switch const&>(*ex);
			out << "select(" << endl();
			indent();
			for( auto & s : es.expr_list ) {
				validate(s);
				out << indent_str();
				if( s.cond ) {
					dump_expression( s.cond.get() );
					out << " ? ";
				}
				dump_expression( s.value.get() );
				out << "," << endl();
			}
			unindent();
			out << indent_str() << ")";
			break;
		}
			
		case expression::f_subscript: {
			auto & es = static_cast<expr_subscript const&>(*ex);
			dump_expression( es.base.get() );
			if( es.optional ) {
				out << "?";
			}
			out << "#(";
			dump_expression( es.offset.get() );
			out << ")";
			break;
		}
			
		case expression::f_block: {
			auto & b = static_cast<expr_block const&>(*ex);
			dump_statement_block( b.block.get() );
			if( b.result ) {
				dump_expression( b.result.get() );
			}
			break;
		}
			
		case expression::f_bind: {
			auto & eb = static_cast<expr_bind const&>(*ex);
			out << "bind" << (eb.clone ? "_clone" :"") << "( ";
			dump_expression( eb.context.get() );
			out << ", ";
			dump_expression( eb.unbound.get() );
			out << ")";
			break;
		}
		
		case expression::f_transform: {
			auto & et = static_cast<expr_transform const&>(*ex);
			switch( et.transform )
			{
				case expr_transform::t_deref:
					out << "⋆";
					dump_expression( et.arg.get() );
					break;
			
				case expr_transform::t_drop_lvalue:
					out << "drop_lvalue( ";
					dump_expression( et.arg.get() );
					out << ") ";
					break;
					
				case expr_transform::t_shared_expr:
					out << "shared_expr(";
					dump_expression( et.arg.get() );
					out << ")" ;
					break;
					
				case expr_transform::t_promote:
					out << "↑";
					dump_expression( et.arg.get() ); //TODO: force show types (important here!)
					break;
					
				case expr_transform::t_loss:
					out << "↓";
					dump_expression( et.arg.get() ); //TODO: force show types (important here!)
					break;
					
				case expr_transform::t_make_optional:
					out << "optional(";
					dump_expression( et.arg.get() );
					out << ")";
					break;
					
				case expr_transform::t_deref_optional:
					out << "unopt(";
					dump_expression( et.arg.get() );
					out << ")";
					break;
					
				case expr_transform::t_addr_of:
					out << "addr_of(";
					dump_expression( et.arg.get() );
					out << ")";
					break;
					
				case expr_transform::t_code_val:
					out << "code_val(";
					dump_expression( et.arg.get() );
					out << ")";
					break;
				
				case expr_transform::t_char_val:
					out << "char_val(";
					dump_expression( et.arg.get() );
					out << ")";
					break;
			}
			break;
		}
		
		case expression::f_field: {
			auto & ef = static_cast<expr_field const&>(*ex);
			dump_expression( ef.base.get() );
			if( ef.optional ) {
				out << "?";
			}
			if( ef.field_index ) {
				out << "#" << *ef.field_index;
				if (ef.field.size()) {
					out << "/*" << ef.field << "*/";
				}
			} else if( ef.sref ) {
				out << "." << ef.sref->symbol;
			} else {
				out << "." << ef.field;
			}
			
			if( ef.param ) {
				out << unicode::encode8(sym_open_param);
				dump_type_spec( ef.param.get() );
				out << unicode::encode8(sym_close_param);
			}
			
			break;
		}
		
		case expression::f_special_field: {
			auto & sf = static_cast<expr_special_field const&>(*ex);
			switch( sf.field )
			{
				case expr_special_field::f_array_size:
					dump_expression( sf.base.get() );
					out << ".size";
					break;
				case expr_special_field::f_optional_set:
					out << "have(";
					dump_expression( sf.base.get() );
					out << ")";
					break;
				case expr_special_field::f_contextual_context: {
					dump_expression( sf.base.get() );
					out << ".context";
					break;
				}
				case expr_special_field::f_contextual_unbound: {
					dump_expression( sf.base.get() );
					out << ".unbound";
					break;
				}
			}
			break;
		}
		
		case expression::f_temp: {
			auto & et = static_cast<expr_temp const&>(*ex);
			omit_type = true;
			out << "temp｢";
			dump_type_ref( ex->type );
			out << "｣(";
			switch( et.init ) {
				case expr_temp::i_none: out <<"none"; break;
				case expr_temp::i_default: out << "default"; break;
				case expr_temp::i_intrinsic: dump_expression( et.intrinsic_value.get() );
			}
			out << ")";
			break;
		}
		
		case expression::f_new_func: {
			auto & nf = static_cast<expr_new_func const&>(*ex);
			out << "new( ";
			show_type( nf.new_type, nf.new_spec_type.get() );
			out << " )";
			break;
		}
		
		case expression::f_conditional: {
			auto & ec = static_cast<expr_conditional const&>(*ex);
			switch( ec.op ) {
				case expr_conditional::o_optional_or:
					dump_expression( ec.cond.get() );
					out << " | ";
					dump_expression( ec.b.get() );
					break;
				case expr_conditional::o_optional_cond:
					dump_expression( ec.cond.get() );
					out << " ? ";
					dump_expression( ec.a.get() );
					break;
			}
			break;
		}
		
		case expression::f_binary: {
			auto & eb = static_cast<expr_binary const&>(*ex);
			switch( eb.op ) {
				#define OP( part ) \
					case expr_binary::o_##part: \
						out << "_" #part; \
						break;
				OP(add)
				OP(sub)
				OP(mul)
				OP(div)
				
				OP(mod)
				
				OP(bit_shl)
				OP(bit_shr)
				
				OP(or)
				OP(and)
				OP(xor)
				OP(and_not)
				OP(nor)
				OP(nand)
				
				OP(eq)
				OP(neq)
				OP(lt)
				OP(lte)
				OP(gt)
				OP(gte)
				#undef OP
			}
			out << "(";
			dump_expression( eb.a.get() );
			out << ",";
			dump_expression( eb.b.get() );
			out << ")";
			break;
		}
		
		case expression::f_unary: {
			auto & eb = static_cast<expr_unary const&>(*ex);
			switch( eb.op ) {
				#define OP( part ) \
					case expr_unary::o_##part: \
						out << "_" #part; \
						break;
				OP(neg)
				OP(abs)
				
				OP(not)
				#undef OP
			}
			out << "(";
			dump_expression( eb.arg.get() );
			out << ")";
			break;
		}
		
		case expression::f_current: {
			auto & c = static_cast<expr_current const&>(*ex);
			switch( c.what ) {
				case expr_current::w_this: out << "$this"; break;
				case expr_current::w_context: out << "$context(" << c.depth << ")"; break;
			}
			break;
		}
		
		case expression::f_cast: {
			auto & c = static_cast<expr_cast const &>(*ex);
			dump_expression( c.source.get() );
			out << " : ";
			dump_type_spec( c.spec_type.get() );
			break;
		}
		
		case expression::f_compat: {
			auto & c = static_cast<expr_compat const &>(*ex);
			out << "compat<";
			dump_type_ref( ex->type );
			out << ">( ";
			dump_expression( c.source.get() );
			out << ")";
			break;
		}
		
		case expression::f_type: {
			auto & t = static_cast<expr_type const &>(*ex);
			out << "type｢ ";
			if (t.inner_type.is_defined() ) {
				dump_type_ref( t.inner_type );
			} else {
				dump_type_spec( t.inner_type_spec.get() );
			}
			out << " ｣";
			break;
		}
		
		case expression::f_tag: {
			auto & t = static_cast<expr_tag const &>(*ex);
			out << "string_tag( ";
			dump_expression( t.value.get() );
			out << " )";
			break;
		}
	}
	
	if( !omit_type ) {
		if( show_all_types && ex->type.is_defined() ) {
			out << " : ";
			dump_type_ref( ex->type );
		}
	}
	
	if( SHOW_COMPLETENESS && !ex->is_resolved() ) {
		out << " /*UNRESOLVED*/";
	}
	if( list_close )
		out << " )";
}

void dump::dump_tuple( expr_with_sub const * ex, char open, char close, char sep ) {
	CHECK_NULL(ex,"tuple")
	
	out << open << " ";
	for( size_t i = 0; i < ex->sub.size(); i++ ) {
		if( i )
			out << sep << " ";
		if( ex->sub[i].name.size() ) {
			out << ex->sub[i].name << " = ";
		}
		dump_expression( ex->sub[i].expr.get(), ef_isolated );
	}
	if( ex->sub.size() )
		out << " ";
	out << close;
}

void dump::dump_funcdefn( expr_funcdefn const & ef ) {
	show_type( ef.type, ef.func_type.get() );
	for( auto & vd : ef.arg_defaults ) {
		out << endl() << indent_str() 
			<< "[" << vd.first << "] |= ";
		dump_expression( vd.second.get(), ef_isolated );
	}
	out << " ";
	if( ef.clone ) {
		out << "clone ";
	}

#if 0
	auto & refs = ef.block->symbols->get_context_vars();
	if( refs.size() ) {	
		out << "context_vars {";
		indent();
		for( auto & ref : refs ) {
			out << endl() << indent_str();
			dump_declaration( ref.get() );
		}
		unindent();
		out << endl() << indent_str() << "} ";
	}
#endif
	
	dump_statement_block( ef.block.get() );
}
