void dump::dump_type_spec( const type_spec * vt ) {
	CHECK_NULL( vt, "type_spec" )
	
	if( vt->params.size() ) {
		out << "｢ ";
		for( size_t i=0; i < vt->params.size(); ++i ) {
			if( i ) {
				out << ", ";
			}
			out << vt->params[i].name << " : ";
			auto now = out.tellp();
			dump_type_spec( vt->params[i].type.get() );
			if( now != out.tellp() ) {
				out << " ";
			}
		}
		out << "｣ ";
	}
	
	//show all markers/labels in alphabetical order (predictable for tests)
	//except fundamental, it always comes first
	std::vector<std::string> markers;
	
	bool no_fundamental = true;
	for( auto & p : vt->get_parts() ) {
		std::string mark;

		//TODO: should not be using get<> here, but grabbing value from part somehow
		switch( p.type ) {
			case pt_fun_class: {
				no_fundamental = false;
				switch( vt->get<pt_fun_class>() ) {
					case intr_type::f_char:
						out << "char";
						break;

					case intr_type::f_tuple:
						dump_intr_type_tuple_inner( "[", "]", vt );
						break;

					case intr_type::f_value_list:
						dump_intr_type_tuple_inner( "(", ")", vt );
						break;
						
					case intr_type::f_function:
						STATE_CHECK( vt->sub.size() == 2 || vt->sub.size() == 1 );
						dump_type_spec(vt->sub[0].type.get());
						out << " -> ";
						if( vt->sub.size() > 1 ) {
							dump_type_spec(vt->sub[1].type.get());
						} else {
							out << "$implicit";
						}
						break;
						
					case intr_type::f_rational:
						out << "rational";
						break;
						
					case intr_type::f_integer:
						out << "integer";
						break;
						
					case intr_type::f_binary:
						out << "binary";
						break;
						
					case intr_type::f_float:
						out << "float";
						break;
						
					case intr_type::f_boolean:
						out << "boolean";
						break;
				
					case intr_type::f_array:
						out << "array";
						break;
						
					case intr_type::f_service:
						out << "service";
						break;
					case intr_type::f_class:
						out << "class";
						break;
					case intr_type::f_closure:
						out << "closure";
						break;
					case intr_type::f_module:
						out << "module";
						break;
						
					case intr_type::f_contextual:
						out << "contextual";
						break;
						
					case intr_type::f_compat:
						//output as compat_type field
						no_fundamental = true;
						break;
						
					case intr_type::f_type:
						out << "type";
						break;
				}
				break;
			}
			
			case pt_fun_prototype: {
				if( !no_fundamental )
					out << " ";
				no_fundamental = false;
					
				if( !vt->get<pt_fun_prototype>()->has_distinct_name() ) {
					out << "prototype(" << get( vt->get<pt_fun_prototype>() ) << ")";
				} else {
					out << std::string( "prototype(" ) << vt->get<pt_fun_prototype>()->get_distinct_name() << ")";
				}
				break;
			}

			case pt_context: {
				//TODO: order/spaceing?
				out << " context(";
				dump_extr_type( vt->get<pt_context>() );
				out << ")";
				break;
			}
			
			case pt_context_name: {
				out << " context(";
				out << vt->get<pt_context_name>();
				out << ")";
				break;
			}
			
			#define BOOL_SPEC(s,m) case s: \
				if( vt->get_part_value<bool>(s) ) { \
					mark = m; \
				} else { \
					mark = "!" m; \
				}  \
				break;
			BOOL_SPEC( pt_signed, "signed" );
			BOOL_SPEC( pt_selectable, "selectable" );
			BOOL_SPEC( pt_member, "member" );
			BOOL_SPEC( pt_lossy, "lossy" );
			BOOL_SPEC( pt_lvalue, "lvalue" );
			BOOL_SPEC( pt_optional, "optional" );
			BOOL_SPEC( pt_cvt_optional, "convert_optional" );
			BOOL_SPEC( pt_literal, "literal" );

			case pt_reference: {
				switch( vt->get<pt_reference>() ) {
					case extr_type::r_value: mark = "value"; break;
					case extr_type::r_shared: mark = "shared"; break;
					case extr_type::r_value_ptr: mark = "value_ptr"; break;
				}
				break;
			}
				
			case pt_function_access: {
				switch( vt->get<pt_function_access>() ) {
					case intr_type_function::a_bound: break;
					case intr_type_function::a_raw: mark = "raw"; break;
					case intr_type_function::a_unbound: mark = "unbound"; break;
					case intr_type_function::a_construct: mark = "construct"; break;
				}
				break;
			}
			
			case pt_function_throw: {
				if (!vt->get<pt_function_throw>() ) {
					mark = "no_throw";
				}
				break;
			}
			
			case pt_pack: {
				switch( vt->get<pt_pack>() ) {
					case intr_type_tuple::p_normal: break;
					case intr_type_tuple::p_abi: mark = "abi_pack"; break;
					case intr_type_tuple::p_byte: mark = "byte_pack"; break;
				}
				break;
			}
			
			case pt_param: {
				mark = "%" + vt->get<pt_param>();
				break;
			}
			
			case pt_symbol: {
				auto & tss = boost::get<type_spec_symbol>( p.value );
				for (size_t i=0; i < tss.parts.size(); ++i) {
					if (i)
						mark += ".";
					mark += tss.parts[i];
				}
				break;
			}
			
			case pt_data_bitsize: {
				char bits[128];
				sprintf( bits, "%dbit", vt->get<pt_data_bitsize>() );
				mark = bits;
				break;
			}

			case pt_precision_variant: {
				switch( vt->get<pt_precision_variant>() ) {
					case -1: mark = "low"; break;
					case 0: mark = "normal"; break;
					case 1: mark = "high"; break;
					default: {
						char bits[128];
						sprintf( bits, "??precision(%d)", vt->get<pt_precision_variant>() );
						mark = bits;
					}
				}
				break;
			}
			
			case pt_compat_type: {
				switch( vt->get<pt_compat_type>() ) {
					case intr_type_compat::abi_var_args: mark = "abi_var_args"; break;
					case intr_type_compat::string_tag: mark = "string_tag"; break;
				}
				break;
			}
						
			case pt_tuple: {
				mark = "tuple";
				break;
			}
		}
		
		//add sub-params to marker (or output if not a mark)
		if( p.sub.size() ) {
			std::string sub( "｢ " );
			for( size_t i=0; i < p.sub.size(); ++i ) {
				if( i ) {
					sub += ", ";
				}
				auto q = get( p.sub[i] );
				if( q == "" ) {
					q = "any";
				}
				sub += q;
			}
			sub += " ｣";
			if( mark.size() ) {
				mark += sub;
			} else {
				out << sub;
			}
		}
		
		if( p.node_expr ) {
			std::string sub("(");
			sub += node_dump::dump_flat(*p.node_expr);
			sub += ")";
			if( mark.size() ) {
				mark += sub;
			} else {
				out << sub;
			}
		}
		
		if( p.expr ) {
			mark += get(p.expr);
		} else if( p.node_expr ) {
			//TODO: show node
		}
		
		if( mark.size() ) {
			markers.push_back( mark );
		}
		
	}
		
	std::sort( markers.begin(), markers.end() );
	for( size_t i=0; i < markers.size(); ++i ) {
		if( i || !no_fundamental )
			out << " ";
		out << markers[i];
	}
}
