//evaluation of builtins with function syntax
#include "lang/expression_typer/common.hpp"

#define SHOW_DEBUG 0
#include "util/debug.hpp"

namespace leaf {
namespace expression_typer {

namespace {
} //eon anon

result process_funccall_ctor( shared_ptr<expr_funccall> expr, context ctx ) {
	//if it already has a context it means the construction aspect was resolved already
	if( expr->context ) {
		return result();
	}
	
	auto func_sym = expr->func;	
	auto ft = func_sym->type.extr().if_cast<intr_type_function>();
	if( !ft || ft->access != intr_type_function::a_construct) {
		return result();
	}
	
	DEBUG( "CTOR-FUNC", dump::get(expr->func->type), dump::get(ft->construct_result),
		ft->is_complete() );
	if( !ft->is_complete() ) {
		STMT_CHECK( *expr, !ctx.no_incomplete(), "incomplete-typing" );
		return expr;
	}
	
	COND_PROC_EXPRESSION( expr->args, ctx, expr );
	STMT_CHECK( *expr, expr->args->form == expression::f_value_list );

	//TODO: copied from process_type_class, likely a high-level instantation op would be best
	auto create = make_shared<expr_temp>();
	create->init = expr_temp::i_none;
	create->src_loc = expr->src_loc;
	create->type = ft->construct_result.is_defined() ? ft->construct_result : 
		extr_type(ft->context.extr().intr); //TODO: questionable stripping
	create->mark_resolved();
		
	//share it for initialization and return
	auto shr = ctx.applicator().create_shared(create);
		
	auto xblock = make_shared<expr_block>();
	xblock->block = make_shared<statement_block>(scope_style_t::imperative);
	xblock->result = shr;
	
	//convert to a suitable context call type
 	auto target = extr_type(ft->context.extr().intr);
 	auto cvt = ctx.converter().convert(target, create->type);
 	STMT_CHECK( *expr, cvt.okay(), cvt.explain_error() );
 	auto cvt_shr = ctx.applicator().apply( cvt, shr );
		
	//add call to the constructor
	expr->context = shr;
	auto es = make_shared<expression_statement>();
	es->expr = expr;
	xblock->block->list.push_back(es);

	DEBUG( "CTOR", dump::get(xblock));
	ctx.rewrite(expr, xblock);
	return xblock;
}

} //eon expression_typer
} //eon leaf
