#include "lang/language.hpp"
#include "lang/expression_visitor.hpp"

#define SHOW_DEBUG 0
#include "util/debug.hpp"

namespace leaf
{

expression_transformer::expression_transformer( bool no_default_pass )
	: no_default_pass( no_default_pass )
{
}

shared_ptr<expression> expression_transformer::apply_expression( shared_ptr<expression> in )
{
	DEBUG( "APPLY_EXPR" );
	switch( in->form ) {
		#define EXPRESSION_FORM(form) \
			case expression::f_##form: \
				return apply_##form( std::static_pointer_cast<expr_##form>(in) );
		EXPRESSION_FORMS
		#undef EXPRESSION_FORM
	}
	
	STATE_FAIL( "unexpected" );
}

#define EXPRESSION_FORM(form) \
	shared_ptr<expression> expression_transformer::apply_##form( shared_ptr<expr_##form> in ) \
	{ \
		if( no_default_pass ) { STMT_FAIL( *in, std::string("unsupported: " ) + typeid(*in).name() ); } \
		else { return in; }  \
	}
EXPRESSION_FORMS
#undef EXPRESSION_FORM

} //eon leaf