#include "lang/module_loader.hpp"
#include "lang/serial.hpp"

#include <fstream>

#include <boost/filesystem.hpp>
namespace fs = boost::filesystem;

namespace leaf {

module_loader::module_loader( std::string const & leaf_path ) : 
	leaf_path( leaf_path ) {
#if defined(IS_DARWIN)
	dyn_ext = ".dylib";
	obj_ext = ".o";
	leaflib_ext = ".lo";
	exe_ext = "";
#else
	dyn_ext = ".so";
	obj_ext = ".o";
	leaflib_ext = ".lo";
	exe_ext = "";
#endif
}

std::string module_loader::resolve_dynlink( std::string const & mod_name ) {
	auto path = leaf_path + "/libs/" + mod_name + dyn_ext;
	//TODO: a system resolution mechanism/configuration path should be used
	if( !fs::exists( path ) ) {
		path = std::string( "/usr/lib/" ) + platform::triple + "/lib" + mod_name + dyn_ext;
	}
	if( !fs::exists( path ) ) {
		path = std::string( "/usr/lib/lib" ) + mod_name + dyn_ext;
	}
	return path;
}

std::string module_loader::resolve_dynload( std::string const & mod_name ) {
	return leaf_path + "/libs/" + mod_name + dyn_ext;
}

std::string module_loader::resolve_static_core( std::string const & part ) {
	return leaf_path + "/leaf_" + part + obj_ext;
}

std::string module_loader::resolve_dyn_core( std::string const & part ) {
	return leaf_path + "/libleaf_" + part + dyn_ext;
}

std::string module_loader::resolve_import( std::string const & part ) {	
	return leaf_path + "/libs/" + part + leaflib_ext;
}

std::string module_loader::obj_name( std::string const & base ) {
	return base + obj_ext;
}

std::string module_loader::dyn_name( std::string const & base ) {
	return base + dyn_ext;
}

std::string module_loader::exe_name( std::string const & base ) {
	return base + exe_ext;
}

std::string module_loader::leaflib_name( std::string const & base ) {
	return base + leaflib_ext;
}
}
