#include "lang/statement_block.hpp"
#include "lang/scope.hpp"
#include "lang/init_statement.hpp"
#include "lang/serial.hpp"

namespace leaf
{

statement_block::statement_block( scope_style_t scope_style )
	: symbols( scope::create( scope_style ) ) {
	flow = 0;
}

statement_block::~statement_block() {
}

void statement_block::validate(util::validation_context & vctx) const {
	STATE_CHECK(symbols);
	for( auto & s : list ) {
		STATE_CHECK( s );
		s->validate(vctx);
	}
}

void statement_block::update_flow() {
	flow = 0;
	for( auto & s : list )
		flow |= s->get_flow();
}

extant_level statement_block::get_extant() const {
	auto level = extant_level::complete;
	for( auto & s : list ) {
		level = std::min( level, s->get_extant() );
	}
	return level;
}

shared_ptr<statement_block> statement_block::pre_clone() const {
	auto ret = make_shared<statement_block>( symbols->get_scope_style() );
	for( auto & s : list ) {
		ret->list.push_back( s->pre_clone() );
	}
	ret->src_loc = src_loc;
	return ret;
}

scope_style_t statement_block::get_scope_style() const {
	return symbols->get_scope_style();
}

statement_block::statement_block( serial_in & si ) {
	symbols = si.read_scope_ref();
	si.read( reference_text );
	
	size_t len;
	si.read( len );
	for( size_t i=0; i < len; ++i ) {
		list.push_back( si.read_statement_ref() );
	}
}

void statement_block::write( serial_out & so ) {
	so.write_ref( symbols );
	so.write( reference_text );
	
	size_t len = list.size();
	so.write( len );
	for( auto & st : list ) {
		so.write_ref( st );
	}
}

} //eon leaf
