#include "lang/language.hpp"
#include "lang/statement_visitor.hpp"
#include "lang/expression_visitor.hpp"

#define SHOW_DEBUG 0
#include "util/debug.hpp"

namespace leaf
{

statement_transformer::statement_transformer( bool no_default_pass )
	: no_default_pass( no_default_pass )
{
}

shared_ptr<statement> statement_transformer::apply_statement( shared_ptr<statement> in,
	shared_ptr<scope> sc)
{
	DEBUG("APPLY_STATEMENT");
	switch( in->statement_form() ) {
		#define STATEMENT_FORM(form) \
			case sf_##form##_statement: \
				return istmt_##form( std::static_pointer_cast<form##_statement>(in), sc );
		STATEMENT_FORMS
		#undef STATEMENT_FORM
	}
	
	STATE_FAIL( "unexpected" );
}

void statement_transformer::apply_statement_block( shared_ptr<statement_block> block )
{
	DEBUG("APPLY_BLOCK" );
	for( auto & st : block->list ) {
		st = apply_statement(st, block->symbols );
	}
}

#define STATEMENT_FORM(form) \
	shared_ptr<statement> statement_transformer::istmt_##form( shared_ptr<form##_statement> in,  \
		shared_ptr<scope> ) { \
		if( no_default_pass ) { STMT_FAIL( *in, std::string( "unsupported: ") + typeid(*in).name() ); } \
		else { return in; }  \
	}
STATEMENT_FORMS
#undef STATEMENT_FORM

///////////////////////////////////////////////////////////
// stat_expr_transformer
stat_expr_transformer::stat_expr_transformer( expression_transformer & etrans )
	: statement_transformer( false ) //TODO: set back to true, it catches important errors
	, etrans( etrans )
{
}

shared_ptr<statement> stat_expr_transformer::istmt_init( shared_ptr<init_statement> in, 
	shared_ptr<scope> )
{
	DEBUG("APPLY_INIT");
	in->value = etrans.apply_expression( in->value );
	return in;
}

shared_ptr<statement> stat_expr_transformer::istmt_expression( shared_ptr<expression_statement> in, 
	shared_ptr<scope> )
{
	DEBUG("APPLY_EXPRESSION");
	in->expr = etrans.apply_expression( in->expr );
	return in;
}

shared_ptr<statement> stat_expr_transformer::istmt_type( shared_ptr<typedef_statement> in, 
	shared_ptr<scope> )
{
	return in;
}

shared_ptr<statement> stat_expr_transformer::istmt_var( shared_ptr<var_statement> in, shared_ptr<scope> )
{
	return in;
}

} //eon leaf