//included in ../type_identifier.cpp

/**
	Adds defaults to the spec.  This is done in situations where the input spec is incomplete, but was
	meant to be a complete spec by the user (such as var inference and constraint)
*/
void complete_defaults( context const & /*ctx*/, type_spec & ts ) {
	DEBUG( "Defaults", dump::get(ts), ts.has_known_intrinsic() );
	if( !ts.has_known_intrinsic() ) {
		return;
	}

	auto fc = ts.get_any_fun_class();
	switch( fc ) {
		case intr_type::f_integer:
			ts.default_set<pt_data_bitsize>(platform::natural_integer_size);
			break;
	
		case intr_type::f_float:
			if( !ts.has<pt_data_bitsize>() ) {
				ts.default_set<pt_precision_variant>(platform::natural_float_precision_variant);
			}
			break;
			
		case intr_type::f_function:
			if( ts.get<pt_member>(false) ) {
				ts.default_set<pt_function_access>(intr_type_function::a_unbound);
			} else if( ts.has<pt_context_name>() || ts.has<pt_context>() ) {
				ts.default_set<pt_function_access>(intr_type_function::a_unbound);
			} else {
				ts.default_set<pt_function_access>(intr_type_function::a_bound);
			}
			break;
	
		case intr_type::f_tuple:
			ts.default_set<pt_pack>(intr_type_tuple::p_normal);
			break;
			
		default:
			break;
	}
	
	ts.default_set<pt_lossy>(false);
	ts.default_set<pt_lvalue>(false);
	ts.default_set<pt_optional>(false);
	ts.default_set<pt_cvt_optional>(false);
	ts.default_set<pt_reference>(extr_type::r_value);
}

void apply_infer( context const & ctx, type_spec & ts ) {
	auto fc = ts.get_any_fun_class();

	//rationals don't exist as actual variables (literal declarations exist however)
	if( fc == intr_type::f_rational ) {
		if( ts.has<pt_data_bitsize>() ) {
			ts.set<pt_fun_class>( intr_type::f_integer );
			ts.clear<pt_data_bitsize>();
		} else {
			ts.set<pt_fun_class>( intr_type::f_float );
		}
		ts.clear<pt_fun_prototype>();
	} else if( fc == intr_type::f_function ) {
		auto context = ts.if_get<pt_context>();
		if( context ) {
			auto & ft = (*context).intr->cast<intr_type_custom>();
			if( !ft.has_distinct_name() && 
				!ctx.get_decl_scope()->is_base_reachable( ft.context) ) {
				//infer bound version of any function belonging to a scope not avaiable in the declaration scope
				ts.clear<pt_context>();
				ts.set<pt_function_access>(intr_type_function::a_bound);
			}
		}
	}
	
	ts.clear<pt_lvalue>();
	ts.clear<pt_literal>();
	
	for( auto & s : ts.sub ) {
		auto c = make_shared<type_spec>(*s.type);
		apply_infer( ctx, *c );
		s.type = c;
	}
}
