//included in ../type_identifier.cpp

/**
	Instantiates a parametric type with the provided parameters.
*/
resolve_t instantiate( context const & ctx, type_ref pt, param_map_t const & params ) {
	
	if( !pt.contains_parametric() ) {
		return pt;
	}
	
	switch( pt.type() ) {
		case type_ref::t_undefined:
			STATE_FAIL( "unexpected-parametric-type-ref" );

		case type_ref::t_spec: {
			auto r = expand( ctx, pt.spec(), params, df_instantiate );
			if (!r.okay()) {
				return resolve_t(r);
			}
			
			auto q = determine( ctx, r.result );
			if(!r.okay()) {
				return resolve_t(r);
			}
			
			return q.result;
		}
		
		case type_ref::t_extr:
			switch( pt.extr().intr->get_fun_class() ) {
					
				case intr_type::f_tuple: {
					auto & tt = pt.extr().cast<intr_type_tuple>();
					auto dup = tt.clone_derived();
					for( auto & sr : dup->sub ) {
						auto r = instantiate( ctx, sr.type, params );
						if(!r.okay()) {
							return resolve_t(r);
						}
						sr.type = r.result;
					}
					return resolve_t(extr_type(dup));
				}
					
				default:
					STATE_FAIL( "unexpected-parametric-fundamental" );
			}
	}
	
	STATE_FAIL( "unreachable" );
}

resolve_intr_type_t instantiate( context const & ctx, shared_ptr<intr_type const> & pt, 
	param_vector_t const & params ) {
	
	STATE_CHECK( pt->contains_parametric() );
	STATE_CHECK( params.size() == pt->params.size() );

	//look for exisintg instance first, multiple instances must be the same object, otherwise
	//other logic becomes harder
	auto cur = pt->if_find_instance( params );
	if( cur ) {
		//DEBUG( "FOUND INSTANCE", dump::get(cur) );
		return cur;
	}
	
	shared_ptr<intr_type> dup;
	
	//DEBUG( "INSTANTIATE", dump::get( pt ) );
	param_map_t pmap;
	for( size_t i = 0; i < params.size(); ++i ) {
		//DEBUG( "PARAM", pt->params[i].name, dump::get( params[i] ) );
		pmap[pt->params[i].name] = params[i];
	}
	
	if( pt->is<intr_type_tuple>() ) {
		auto & tt = pt->cast<intr_type_tuple>();
		auto tdup = tt.clone_derived();
		for( auto & sr : tdup->sub ) {
			auto ir = instantiate( ctx, sr.type, pmap );
			if(!ir.okay()) {
				return resolve_intr_type_t(ir);
			}
			sr.type = ir.result;
		}
		
		dup = tdup;
	
	} else if( pt->is<intr_type_array>() ) {
		auto & tw = pt->cast<intr_type_array>();
		auto tdup = tw.clone_derived();
		auto ir = instantiate( ctx, tdup->sub, pmap );
		if( !ir.okay() ) {
			return resolve_intr_type_t(ir);
		}
		tdup->sub = ir.result;
		dup = tdup;
		
	} else if( pt->is<intr_type_custom>() ) {
		dup = pt->clone();
	
	} else {
		STATE_FAIL( "invalid-parameterized-type" );
	}
	
	for( size_t i = 0; i < params.size(); ++i ) {
		dup->applied_params.push_back( params[i] );
	}
	
	dup->clear_param();
	dup->instance_of = pt;
	dup->style = intr_type::s_plain; //loses distinctness
	//DEBUG( "POST-INSTANCE", dump::get(dup), !!dup->instance_of, pt->is_instance());
	
	if( pt->is_instance() ) {
		dup->instance_of = pt->instance_of;
	}
	
	if( dup->is<intr_type_classlike>() ) {
		class_typer::instantiate( std::static_pointer_cast<intr_type_classlike>(dup), ctx );
	}
	pt->instances.push_back( dup );
	return resolve_intr_type_t(dup);
}

