#include "parser/dump.hpp"
#include "parser/node.hpp"
#include "lang/dump.hpp"

#include <iostream>
#include <sstream>

namespace leaf
{

node_dump::node_dump( std::ostream & out )
	: dump_base( out )
{
}

/*static*/ std::string node_dump::dump_flat( node const & n )
{
	std::ostringstream sout;
	node_dump nd( sout );
	nd.set_flat();
	nd.dump( n );
	return sout.str();
}

void node_dump::dump( node const & n )
{	
	out << indent_str() << "[";
	
	switch( n.form )	{
		case node::f_symbol:
		case node::f_literal_boolean:
			out << n.text;
			break;
			
		case node::f_literal_number:
		{
			auto & ncn = static_cast<node_literal_number const &>(n);
			out << ncn.value.to_string();
			if( ncn.was_float )
				out << "f";
			break;
		}
			
		case node::f_expr_list:
			out << "expr-list";
			break;
			
		case node::f_value_list:
			out << "value-list";
			break;
			
		case node::f_param_list:
			out << "param-list";
			break;
		
		case node::f_tuple:
			out << "tuple";
			break;
			
		case node::f_block:
			out << "block";
			break;
			
		case node::f_funccall:
			out << "funccall";
			break;
		
		case node::f_funcdefn:
			out << "funcdefn";
			break;
		
		case node::f_literal_string:
			out << "\"" << n.text << "\"";
			break;
			
		case node::f_literal_char:
			out << "\'" << n.text << "\'";
			break;
			
		case node::f_type:
			break;
			
		case node::f_named:
			out << "named";
			break;
			
		case node::f_on:
			out << "on";
			break;
			
		case node::f_implied:
			out << "implied";
			break;
			
		case node::f_decl_name:
			out << n.text;
			out << "<>";
			break;
	}

	if( n.param ) {
		out << " [param ";
		leaf::dump d( out );
		d.set_hide_var_spec_no_fundamental( true );
		d.write_type_spec( n.param.parsed_type_spec() );
		out << "]";
	}
	
	for( size_t i=0; i < n.sub.size(); ++ i) {
		out << endl();
		indent();
		dump( *n.sub[i] );
		unindent();
	}
	
	if( n.sub.size() )
		out << endopt() << indent_str() << "]";
	else
		out << "]";
		
	if( n.type ) {
		out << " : ";
		leaf::dump d( out );
		d.set_hide_var_spec_no_fundamental( true );
		d.write_type_spec( n.type.parsed_type_spec() );
	}
	
	for( auto anno : n.annotations ) {
		out << " @" << anno.first  << " ";
		dump( *anno.second );
	}
}

} //eon leaf