#include <boost/regex.hpp>

#define SYMBOL_REGEX_CHAR "[a-zA-Z0-9_]"
#define SYMBOL_REGEX_START_CHAR "([a-zA-Z]|⁑)"
#define SYMBOL_REGEX SYMBOL_REGEX_START_CHAR SYMBOL_REGEX_CHAR "*"

#define TYPE_SYMBOL_REGEX_CHAR "[a-zA-Z0-9_.]"
#define TYPE_SYMBOL_REGEX_START_CHAR "([a-zA-Z_]|⁑)"
#define TYPE_SYMBOL_REGEX TYPE_SYMBOL_REGEX_START_CHAR TYPE_SYMBOL_REGEX_CHAR "*"

#define EXCLUDE_SYMBOL_REGEX_CHAR "(?!" SYMBOL_REGEX_CHAR ")"

#define OP_REGEX_MULTI "[:=><\\+\\/\\-\\*\\^\\.\\?@!#%\\|$]+"
//TODO: replace with suitable ranges for punctuation/comparision, etc.
//TODO: get proper unicode regexes! the below doesn't work with [...]
#define OP_REGEX_SINGLE "(↛|∨|∧|↮|↓|↑|¬)"

namespace leaf_parse
{
	// consume all compounding operator symbols into one, this prevents visual ambiguity and 
	// allows for operator extension later.
	// non-compounding operators will never compound
	const boost::regex re_operator( "(" OP_REGEX_MULTI ")|([\\[\\]\\(\\)\\{\\}])"
		"|(" OP_REGEX_SINGLE ")"
		"|((or|and|xor|not|and_not|nor|nand|trap)" EXCLUDE_SYMBOL_REGEX_CHAR ")"  );
		
	const boost::regex re_symbol( SYMBOL_REGEX );
	//specifically disallow leading/trailing dot (may lead to ambiguity with other operators)
	const boost::regex re_number( "[0-9_]+(\\.[0-9_]+)?([eE][+-]?[0-9]+)?" );
	const boost::regex re_base_number( "0([xbo])([0-9a-fA-F_]+(\\.[0-9a-fA-F_]+)?)" );
	const boost::regex re_bool( "true|false" EXCLUDE_SYMBOL_REGEX_CHAR );
	
	const boost::regex re_special( "(new|on)" EXCLUDE_SYMBOL_REGEX_CHAR );
}
