#include <boost/test/unit_test.hpp>

#include <boost/algorithm/string/replace.hpp>

#include <iostream>

/**
	Abstracts some common behaviour of many of the tests.
*/
struct expect_test
{	
	struct part {
		std::string label, value;
	};
	std::vector<part> parts;
	
	expect_test(){
	}
	void add( std::string const & label, std::string const & value ) {
		part p;
		p.label = label;
		p.value = value;
		parts.push_back(p);
	}
	void add( std::string const & value ) {
		char buf[32];
		sprintf( buf, "%d", int(parts.size()) );
		add( buf, value );
	}
	
	void fail( std::string const & msg ) {
		header( msg );
		BOOST_REQUIRE( false );
	}
	void except( ) {
		fail( std::string("Exception: ") + boost::current_exception_diagnostic_information() );
		throw;
	}
	void check( bool cond, std::string const & msg, char const * prefix = "" ) {
		if( !cond ) {
			fail( std::string( prefix ) + ": " + msg );
		} else {
			BOOST_CHECK(true);
		}
	}
	
	template<typename T,typename U>
	void check_actual( T const & actual, U const & expect, char const * msg = nullptr ) {
		if( actual == expect ) {
			BOOST_CHECK( true );
			return;
		}
			
		header( "mismatched" );
		if(msg ) {
			BOOST_MESSAGE( msg );
		}
		BOOST_MESSAGE( "Expect: " << expect );
		BOOST_MESSAGE( "Actual: " << actual );
		BOOST_CHECK(false);
	}
	
	void header( std::string const & msg ) {
		for( auto & p : parts )
			BOOST_MESSAGE( p.label << ": " << p.value );
		BOOST_MESSAGE( msg );
	}
	
	std::string platform_fix( std::string const & in ) const {
		auto out = in;
		boost::replace_all( out, "%PBIT", "64" );
		boost::replace_all( out, "PBIT", "64bit" ); //TODO: obviously format the platform bit-size
		return out;
	}
};
