/**
	Common expression testing/typing test base
*/
#include "type_converter_common.hpp"

#include "lang/language.hpp"
#include "lang/typer.hpp"

#include "parser/node_parser.hpp"
#include "parser/node_converter.hpp"

#include <iostream>

using namespace leaf;

namespace {

struct expr_tester : public type_converter_tester {

	expr_tester() :
		type_converter_tester( style_runner ) {
	}
	

	void add_var( char const * name, char const* type_code ) {
		auto vs = declaration::create();
		vs->symbol = name;
		vs->form = declaration::f_single;
		
		auto rt = type_identifier::determine( the_context(), parse_spec(type_code) );
		STATE_CHECK( rt.okay(), rt.explain_error() );
		vs->type = rt.result;
		
		sc->add( vs );
	}
	
	void add_multi( char const * name, char const* type_code ) {
		auto vs = declaration::create();
		vs->symbol = name;
		vs->form = declaration::f_multi;
		
		auto rt = type_identifier::determine( the_context(), parse_spec(type_code) );
		STATE_CHECK( rt.okay(), rt.explain_error() );
		vs->type = rt.result;
		
		sc->add( vs );
	}

	shared_ptr<expression> parse_expr( char const * code ) {
		auto src = source::create( code, std::strlen( code ) );

		auto n = node_parser::parse_single( *src );
		return node_converter::expression_convert( *n );
	}
};

} //eon anon
