/**
	Tests actaul conversions of expressions. This builds on the expr_type_test and type_converter_test
	foundations and actually tests the conversion_applicator.
*/
#define SHOW_DEBUG 0
#include "expr_common.hpp"
#include "lang/conversion_applicator.hpp"

BOOST_AUTO_TEST_SUITE( expr_conversion_test )

struct tester : public expr_tester {

	typedef type_converter::convert_t (type_converter::*to_from_func)( type_ref const &,
		type_ref const & );
		
	to_from_func use_from_func;
	
	tester( to_from_func use_from_func ) :
		use_from_func( use_from_func ) {
	}
	tester() :
		use_from_func( &type_converter::convert ) {
	}
	/**
		Type "code" and then converts it to "type". Checks that the resulting type is 
		actually equal to "type".
		
		@param expect_type: Expect equal to this type instead
	*/
	void operator()( char const * code, char const * type, char const * expect_type = nullptr ) {
		expect_test et;
		et.add( "Code", code );
		et.add( "Type", type );
		if( expect_type ) {
			et.add( "ExpectType", expect_type );
		} else {
			expect_type = type;
		}
		
		try {
			auto ex = parse_expr( code );
			typer t(sc);
			ex = t.full_type( ex, 0 );
			
			auto tt = determine_type(type);
			DEBUG( "SDF", dump::get( ex->type ), dump::get( tt ) );
			auto cost = (the_context().converter().*use_from_func)( tt, ex->type );
			auto out_ex = the_context().applicator().apply( cost, ex );
			
			et.check_actual( dump::get(out_ex->type), expect_type );
			
		} catch( ... ) {
			et.except();
		}
	}
};

BOOST_AUTO_TEST_CASE( basic ) {
	tester t;
	t( "5", "integer" );
	t( "5", "float literal" );
}

BOOST_AUTO_TEST_CASE( value_list ) {
	tester t;
	t( "(5)", "integer" );
	t( "(5)", "integer literal" );
	
	t( "(2,3)", "( : integer, : float )" );
	t( "(2,3)", "( : integer literal, : float literal )" );
}

BOOST_AUTO_TEST_CASE( tuple ) {
	tester t;
	t( "[1,2]#0", "integer literal" );
	t( "[1,2]", "[ : float, : integer ] literal" );
	t( "([1,(2)])", "[ : float, : integer ] literal" );
	
	t.add_var( "a", "integer" );
	t( "[a]", "[ : integer ]" );
	
	t.add_defined_type( "point", "[ : integer, : integer ]" );
	t( "[3,4]", "point" );
	t( "[3,4]", "point literal" );
}

BOOST_AUTO_TEST_CASE( retain_literal ) {
	tester t( &type_converter::convert_retain_literal );
	t( "1", "integer", "integer literal" );
	t( "(1)", "float", "float literal" );
	t( "([3,(4)])", "[ : integer, : binary ]", "[ : integer, : binary ] literal" );
}


BOOST_AUTO_TEST_SUITE_END()