#include <boost/test/unit_test.hpp>

#include <deque>
/*
	These are actually tests of C++11. The features are still quite new at this time and I must
	ensure they work correctly in the compiler.
*/

BOOST_AUTO_TEST_SUITE( lambda_test )

struct sample {
	int a;
	
	int simple() {
		auto f = [this]() {
			return a;
		};
		return f();
	}
	
	std::function<int()> get_simple(int o) {
		return [=]() {
			return a + o;
		};
	}
};

BOOST_AUTO_TEST_CASE( simple )
{
	sample s;
	s.a = 15;
	BOOST_CHECK_EQUAL( 15, s.simple() );
	
	auto f = s.get_simple(2);
	s.a = 10;
	BOOST_CHECK_EQUAL( 12, f() );
	
	std::deque<std::function<int()>> all;
	all.push_back( s.get_simple(1) );
	all.push_back( s.get_simple(2) );
	all.push_front( s.get_simple(3) );
	all.push_front( s.get_simple(4) );
	auto q = all.front();
	all.pop_front();
	all.push_back( q );
	
	BOOST_CHECK_EQUAL( 13, all[0]() );
	BOOST_CHECK_EQUAL( 11, all[1]() );
	BOOST_CHECK_EQUAL( 12, all[2]() );
	BOOST_CHECK_EQUAL( 14, all[3]() );
}

struct part {
	int a;
	void set_a( int b ) {
		a = b;
	}
	part() {
		a = 0;
	}
};

struct access {
	std::function<int()> try_it( part p ) {
		return [=]() mutable {
			p.set_a(p.a+10);
			return p.a;
		};
	}
};

BOOST_AUTO_TEST_CASE( access_test )
{
	access a;
	part p;
	auto f = a.try_it(p);
	BOOST_CHECK_EQUAL( 10, f() );
	BOOST_CHECK_EQUAL( 20, f() );
}

BOOST_AUTO_TEST_SUITE_END()