#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_converter_cost )

struct tester_cost : public type_converter_tester
{
	void compare( char const * from, char const * to_a, char const * to_b, std::string const & cfunc ) {
		expect_test et;
		et.add( "From", from );
		et.add( "A", to_a );
		et.add( "Op", cfunc );
		et.add( "B", to_b );
		
		try {
			auto vt_from = determine_type( from );
			auto vt_to_a = determine_type( to_a );
			auto vt_to_b = determine_type( to_b );
			
			auto cost_a = the_context().converter().convert( vt_to_a, vt_from );
			auto cost_b = the_context().converter().convert( vt_to_b, vt_from );
			
			int res = cost_a.compare_cost( cost_b );
			bool cond = false;
			if( cfunc == "=" )
				cond = res == 0;
			else if( cfunc == "<" )
				cond = res < 0;
			et.check( cond, "cost compare" );
		} catch( ... ) {
			et.except();
		}
	}
	
	void same( char const * from, char const * to_less, char const * to_more ) {
		return compare( from, to_less, to_more, "=" );
	}
	void less( char const * from, char const * to_less, char const * to_more ) {
		return compare( from, to_less, to_more, "<" );
	}
};

BOOST_AUTO_TEST_CASE(no_cost)
{
	tester_cost t;
	t.same( "(:integer)", "integer", "(:integer)" );
	t.same( "float lvalue", "float", "float lvalue" );
	t.same( "binary optional", "binary", "binary optional" );
	t.same( "binary optional lvalue", "binary", "binary optional lvalue" );
	t.same( "[]", "[]", "[] optional" );
	t.same( "()->() raw", "()->()", "()->() raw" );
	
	t.same( "[:integer optional,:float]", "[:integer,:float optional]", "[:integer,:float]" );
}

BOOST_AUTO_TEST_CASE( deref )
{
	tester_cost t;
	t.less( "boolean shared", "boolean shared", "boolean" );
	t.less( "boolean shared lvalue", "boolean shared", "boolean" );
	
	t.less( "[:integer,:float shared]", "[:integer,:float shared]", "[:integer,:float]" );
}

BOOST_AUTO_TEST_CASE( rationals )
{
	tester_cost t;
	t.less( "rational 10bit", "integer", "float" );
	t.less( "rational 10bit", "integer", "binary" );
	t.less( "rational 10bit", "binary", "float" );
	//NOTE: preference is now to natural, otherwise no preference
	t.less( "rational 10bit", "integer", "integer 20bit" );
	t.same( "rational 10bit signed", "integer 200bit", "integer 10bit" );
	t.same( "rational 10bit", "integer 200bit", "integer 201bit" );
	t.same( "rational 10bit signed", "integer 20bit", "integer 10bit" );
	
	t.less( "rational 10bit", "binary", "binary 20bit" );
	t.same( "rational 10bit", "binary 200bit", "binary 10bit" );
	t.same( "rational 10bit", "binary 200bit", "binary 201bit" );
	t.same( "rational 10bit", "binary 20bit", "binary 10bit" );
	
	t.less( "rational", "float", "float low" );
	t.less( "rational", "float", "float high" );
	t.same( "rational", "float high", "float low" );
}

BOOST_AUTO_TEST_CASE( integral_to_float )
{
	tester_cost t;
	t.less( "integer 16bit", "integer", "float" );
	t.less( "binary 16bit", "integer", "float" );
	t.less( "integer 16bit", "float", "float high" );
	t.less( "binary 16bit", "float low", "float high" );
}

BOOST_AUTO_TEST_CASE( binary_to_integer )
{
	tester_cost t;
	t.less( "binary 10bit", "integer 16bit", "integer 32bit" );
}

BOOST_AUTO_TEST_CASE( binary_to_multi )
{
	tester_cost t;
	t.less( "binary 10bit", "integer", "float" );
}

BOOST_AUTO_TEST_CASE( promotions )
{
	tester_cost t;
	t.less( "integer 10bit", "integer 20bit", "integer 30bit" );
	t.less( "binary 10bit", "binary 20bit", "binary 30bit" );
	t.less( "float", "float", "float high" );
	t.less( "float low", "float", "float high" );
	t.less( "float low", "float low", "float" );
}

BOOST_AUTO_TEST_CASE( floats )
{
	tester_cost t;
	t.less( "float low lvalue", "float low", "float" );
}

BOOST_AUTO_TEST_CASE( packing )
{
	tester_cost t;
	t.same( "[ : integer optional ]", "[ : integer]", "[:integer optional]" );
	t.less( "( : rational 8bit )", "(:integer)", "(:float)" );
	t.less( "( : rational 1bit, : rational 2bit )", "( : integer, : integer )", "( : float, : float )" );
	
	//the absolute scores of sub-matching is not retained, each part is only valued at -1,0,+1
	t.same( "( : rational 8bit, : rational 8bit )", "(:integer,:float)", "(:integer 128bit,:integer)" );
}

BOOST_AUTO_TEST_SUITE_END()