#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_converter_fixate )

struct tester_fixate : public type_converter_tester
{
	void operator()( char const * orig, char const * expect )
	{
		expect_test ex;
		ex.add( "Orig", orig );
		
		type_converter::resolve_t result;
		try {
			auto vt_orig = determine_type( orig );
			result = the_context().converter().fixate( vt_orig );
		} catch( ... ) {
			ex.except();
		}

		ex.check( result.okay(), result.explain_error() );
		ex.check_actual( dump::get( result.result, 0 ), expect );
	}
	
	void operator()( char const * orig, cerr expect )
	{
		expect_test ex;
		ex.add( "Orig", orig );

		type_converter::resolve_t result;
		try {
			auto vt_orig = determine_type( orig );
			result = the_context().converter().fixate( vt_orig );
		} catch( ... ) {
			ex.except();
		}

		ex.check( result.error == expect, result.explain_error(), "wrong error" );
	}
};

BOOST_AUTO_TEST_CASE( fixate_basic )
{
	tester_fixate t;
	t( "integer", "integer" );
	t( "rational 1bit", "integer" );
	t( "rational 31bit", "integer" );
	t( "rational", "float" );
	
	t( "rational 1000bit", cerr::large_rational_integer );
	
	t( "[]", "[ ]" );
	t( "[:integer]", "[ : integer ]" );
	t( "[:rational 1bit]", "[ : integer ]" );
	t( "(:integer, :rational, :[:rational 2bit,:[]])",
		"( : integer, : float, : [ : integer, : [ ] ] )" );
}

BOOST_AUTO_TEST_SUITE_END()