#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_converter_function_call )

struct tester_call : public type_converter_tester {

	void operator()( char const * sto, char const * sfrom, 
		char const * expect ) {
		expect_test ex;
		ex.add( "To", sto );
		ex.add( "From", sfrom );

		type_converter::convert_t cost;
		try {
			type_ref tog, fromg;
			tog = determine_type( sto, type_identifier::df_allow_param );
			auto & to = tog.extr().cast<intr_type_value_list>();
			fromg = determine_type( sfrom, type_identifier::df_allow_param );
			auto & from = fromg.extr().cast<intr_type_value_list>();

			cost = the_context().converter().convert_function_call( to, from );
		} catch( ... ) {
			ex.except();
		}
		
		ex.check( cost.okay(), cost.explain_error() );
			
		std::ostringstream out;
		if( cost.conversions )
			out << dump::get( cost.conversions );
		ex.check_actual( out.str(), expect );
	}
	
	void operator()( char const * sto, char const * sfrom, 
		cerr expect ) {
		expect_test ex;
		ex.add( "To", sto );
		ex.add( "From", sfrom );

		type_converter::convert_t cost;
		try {
			type_ref tog, fromg;
			tog = determine_type( sto );
			auto & to = tog.extr().cast<intr_type_value_list>();
			fromg = determine_type( sfrom );
			auto & from = fromg.extr().cast<intr_type_value_list>();

			cost = the_context().converter().convert_function_call( to, from );
		} catch( ... ) {
			ex.except();
		}
		
		ex.check( cost.error == expect, cost.explain_error(), "wrong error" );
	}
};

BOOST_AUTO_TEST_CASE(basics) {
	tester_call t;
	//some basics, not trying to reject all conversion just that they work via function calls
	t( "()","()", "c_pack_value" );
	t( "(:integer)", "(:integer shared)", "c_pack_value {[0]->c_deref}" );
	t( "(:float,:integer)", "(:rational,:(:integer))",
		"c_pack_value {[0]->c_fixate(normal float) [1]->c_collapse}" );
}

BOOST_AUTO_TEST_CASE(optional) {
	tester_call t;
	t( "( : integer optional )", "()", "c_pack_value {c_unset_optional}" );
	t( "( : float, :float optional )", "(:rational)", "c_pack_value {[0]->c_fixate(normal float) c_unset_optional}" );
}

BOOST_AUTO_TEST_CASE(instantiate) {
	tester_call t;
	t( "( : (x:integer)->() )", "( : (y)->())", "c_pack_value {[0]->c_param_inst｢( y : integer ) -> ( )｣}" );
	t( "( x : array<:integer:>, f : (x:integer)->()) literal raw",
		"( : array<:integer:> lvalue, : ( y : )->() ) literal", 
		"c_pack_value {x=[0]->c_drop_lvalue f=[1]->c_param_inst｢( y : integer ) -> ( )｣}");

	//inferred return variants
	t( "( : (x:integer)->() )", "( : (y)-> $implicit )", "c_pack_value {[0]->c_param_inst｢( y : integer ) -> ( )｣}" );
	t( "( x : array<:integer:>, f : (x:integer)->()) literal raw",
		"( : array<:integer:> lvalue, : ( y : )-> $implicit ) literal", 
		"c_pack_value {x=[0]->c_drop_lvalue f=[1]->c_param_inst｢( y : integer ) -> ( )｣}");
}

BOOST_AUTO_TEST_CASE(array) {
	tester_call t;
	t( "( : array｢integer｣ )", "( : array｢integer｣ )", "c_pack_value {[0]}" );
	//TODO: t( "( : array｢integer｣ )", "( : array｢char｣ )", cerr::mismatch_fundamental );
}

BOOST_AUTO_TEST_SUITE_END()