#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_converter_match_function )

struct tester : public type_converter_tester {
	
	void operator()( std::vector<size_t> expect, char const * args, std::vector<char const*> select ) {
		expect_test ex;
		ex.add( "Args", args );
		for( auto & s : select ) {
			ex.add( "Candidate", s );
		}
		
		std::vector<size_t> matches;
		
		try {
			auto t_args = determine_type(args,type_identifier::df_allow_param);
			auto t_select = make_shared<intr_type_tuple>();
			for( auto & s : select ) {
				t_select->push_sub( determine_type(s,type_identifier::df_allow_param) );
			}
		
			matches = the_context().converter().match_function( *t_select, *t_args.extr().intr );
		} catch(...) {
			ex.except();
		}
		
		ex.check_actual( matches.size(), expect.size(), "Matches Size" );
		for( size_t i=0; i < expect.size(); ++i ) {
			ex.check_actual( matches[i], expect[i], "Match Index" );
		}
	}
};

BOOST_AUTO_TEST_CASE( basic ) {
	tester t;
	t( {0}, "(:integer)", { "(:integer)->()" } );
	t( {}, "(:integer)", { "(:float)->()" } );
	t( {1}, "(:integer)", { "(:float)->()", "(:integer)->()" } );
	t( {0,2}, "(:integer)", { "(:integer)->()", "(:float)->()", "(:integer)->()" } );
	t( {1}, "(:integer 8bit)", { "(:integer)->()", "(:integer 8bit)->()" } );
}

BOOST_AUTO_TEST_CASE( multi ) {
	tester t;
	t( {0,1}, "(:integer 8bit,:integer 8bit)", { "(:integer,:integer 16bit)->()", "(:integer 8bit,:integer)->()" } );
}

BOOST_AUTO_TEST_CASE( parametric ) {
	tester t;
	t( {0}, "(:integer)", { "(:)->()" } );
	t( {1}, "(:integer)", { "(:)->()", "(:integer)->()" } );
	t( {1}, "(:integer)", { "(:)->()", "(:integer)->$implicit" } );
	t( {0,1}, "(:integer)", { "(:)->()", "(:)->()" } );
	t( {0,2}, "(:integer,:float)", { "(:,:float)->()", "(:,:)", "(:integer,:)->()" } );
	t( {0,2}, "(:integer,:float)", { "(:,:float)->$implicit", "(:,:)", 
		"(:integer,:)->$implicit" } );
}

BOOST_AUTO_TEST_CASE( contextual ) {
	tester t;
	t.add_class( "pine" );
	t( {0}, "()", { "contextual<: pine, ()->() context(pine):>" } );
	t( {1,2}, "(:integer)", { 
		"contextual<: pine, integer :>",
		"(:integer)->(:float)",
		"contextual<:pine, (:integer)->() context(pine):>"
	});
}

BOOST_AUTO_TEST_CASE( array ) {
	tester t;
	t( {}, "(:array｢integer｣)", { "(:array｢float｣) -> ()", "(:array｢char｣) ->()"});
	t( {0}, "(:raw_array｢abi_char｣)", { "(:raw_array｢abi_char｣) -> ()", "(:array｢char｣) ->()"});
}

BOOST_AUTO_TEST_SUITE_END()