#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_identifier_constrain )

struct tester_constrain : public type_converter_tester {
	
	void operator()( char const * spec, char const * assign, 
		char const * expect ) {
		expect_test ex;
		ex.add( "Spec", spec );
		ex.add( "Assign", assign );
		
		type_identifier::resolve_t result;
		
		auto vt_spec = parse_spec( spec );
		auto vt_assign = determine_type( assign );

		try {
			result = type_identifier::constrain( the_context(), *vt_spec, vt_assign );
		} catch( ... ) {
			ex.except();
		}
		
		ex.check( result.okay(), result.explain_error() );
		ex.check_actual( dump::get( result.result ), ex.platform_fix(expect) );
	}
	
	void operator()( char const * spec, char const * assign, cerr expect ) {
		expect_test ex;
		ex.add( "Spec", spec );
		ex.add( "Assign", assign );
		
		auto vt_spec = parse_spec( spec );
		auto vt_assign = determine_type( assign );

		type_identifier::resolve_t result;
		try {
			result = type_identifier::constrain( the_context(), *vt_spec, vt_assign );
		} catch( ... ) {
			ex.except();
		}

		ex.check( result.error == expect, result.explain_error(), "wrong error" );
	}
};

BOOST_AUTO_TEST_CASE( fundamental ) {
	tester_constrain t;
	t( "integer", "integer 8bit", "integer" );
	t( "float", "float high", "float" );
	t( "()->()", "()->() raw", "( ) -> ( )" );
	t( "(:integer)->()", "[ : ()->(), :integer]", "( : integer ) -> ( )" );
}

BOOST_AUTO_TEST_CASE( pack ) {
	tester_constrain t;
	t( "[:integer]", "[:integer] abi_pack", "[ : integer ]" );
}

BOOST_AUTO_TEST_CASE( literal ) {
	tester_constrain t;
	t( "", "integer literal", "integer literal" );
	t( "", "rational", "rational" );
	t( "literal", "boolean", "boolean literal" );
	t( "literal", "[ : float, : integer ]", "[ : float, : integer ] literal" );
	t( "", "[ : rational 1bit, : rational 2bit ] literal", "[ : rational 1bit, : rational 2bit ] literal" );
	
	t.add_defined_type( "point", "[ x : float, y : float ]" );
	t( "literal point", "[ : rational 1bit, : rational 2bit ] literal", "point literal" );
	t( "point", "[ : rational 1bit, : rational 2bit ] literal", "point literal" );
}

BOOST_AUTO_TEST_CASE( selectable ) {
	tester_constrain t;
	//this is actually a bit misleading, the selectable is just ignored since an intrinsic is already known
	t( "integer", "[ : integer, : float] selectable", "integer" );
	t( "()->()", "()->() raw lvalue", "( ) -> ( )" );
	t( "(:integer)->()", "[ : ()->(), : (:float)->(), : (:integer)->()] selectable", "( : integer ) -> ( )" );
	t ("()->() no_throw", "[ : ()->(), : ()->() no_throw ]", "( ) -> ( ) no_throw" );
	t ("(:integer)->() no_throw", "[ : ()->(), : (:integer)->() no_throw literal lvalue raw ]", 
		"( : integer ) -> ( ) no_throw" );
}

BOOST_AUTO_TEST_CASE( tuple ) {
	tester_constrain t;
	
	t( "tuple", "[:integer]", "[ : integer ]" );
	t( "tuple", "integer", cerr::requires_tuple );
}

BOOST_AUTO_TEST_CASE( param ) {
	tester_constrain t;
	
	auto cl = t.add_class( "cone" );
	cl.type->params.push_back( { "T" } );
	
	t( "cone<:any:>", "cone<:integer:>", "^cone｢ integer ｣" );
	t( "cone<:shared:>", "cone<:integer:>", "^cone｢ integer shared ｣" );
	t( "cone<:tuple:>", "cone<:[:integer]:>", "^cone｢ [ : integer ] ｣" );
	t( "cone<:tuple:>", "cone<:integer:>", cerr::requires_tuple );
}

BOOST_AUTO_TEST_CASE( array ) {
	tester_constrain t;
	
	t( "array<:any:>", "array<:integer:>", "^array｢ integer ｣" );
	t( "array<:shared:>", "array<:integer:>", "^array｢ integer shared ｣" );
	t( "array<:tuple:>", "array<:[:integer]:>", "^array｢ [ : integer ] ｣" );
	t( "array<:tuple:>", "array<:integer:>", cerr::requires_tuple );
}

BOOST_AUTO_TEST_SUITE_END()