/**
	Tests the expansion of type specs. There are al imited number of things tested here as the
	other identifier tests were written before and heavily rely on the expand code already.
*/
#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

#include "platform/platform.hpp"
#include <cstdio>
#include <stdarg.h>

BOOST_AUTO_TEST_SUITE( type_identifier_expand )

struct tester : public type_converter_tester
{
	tester() {
	}
	
	void operator()( char const * spec, char const * expect ) {
		expect_test ex;
		ex.add( "Spec", spec );

		type_identifier::resolve_spec_t result;
		try {
			auto vts = parse_spec(spec);
			result = type_identifier::expand( the_context(), *vts );
			STATE_CHECK(result.okay(),result.explain_error());
		} catch(...) {
			ex.except();
		}
	
		leaf::dump d;
		d.set_validate(false);
		d.write_type_spec( *result.result );
		ex.check_actual( d.str(), expect );
	}
	
	void operator()( char const * spec, cerr expect ) {
		expect_test ex;
		ex.add( "Spec", spec );
		
		type_identifier::resolve_spec_t result;
		try {
			auto vt_spec = parse_spec(spec);
			result = type_identifier::expand( the_context(), *vt_spec );
		} catch( ... ) {
			ex.except();
		}
		
		ex.check( result.error == expect, result.explain_error(), "wrong error" );
	}
};

BOOST_AUTO_TEST_CASE( basic ) {
	tester t;
	t( "integer ", "integer" );
	t( "none", cerr::unknown_identifier );
}

BOOST_AUTO_TEST_CASE( parametric ) {
	tester t;
	t( "<:T:> T", "｢ T : ｣ %T" );
	t( "<:T,Q:shared:> [:Q, :T]", "｢ T : , Q : shared ｣ [ : %Q, : %T ]" );
	t( "<:R:> array<:R:>","｢ R : ｣ prototype(array)｢ %R ｣" );
	
	t.add_type_alias( "Ally", "<:X:> X" );
	t( "Ally<:integer:>", "integer" );
	t( "<:Q:> Ally<:Q:>", "｢ Q : ｣ %Q" );
	
	t( "array<:Ally<:integer:>:>", "prototype(array)｢ integer ｣" );
	t( "Ally<:Ally<:array<:Ally<:float:>:>:>:>", "prototype(array)｢ float ｣" );

	t.add_type_alias( "Oop", "<:Q:> Q" );
	t( "<:T:> Ally<:Oop<:T:>:>", "｢ T : ｣ %T" );
	
	t( "<:Q,R,S:> [ : Q shared, : array<:R:>, : Ally<:S:> ]",
		"｢ Q : , R : , S : ｣ [ : %Q shared, : prototype(array)｢ %R ｣, : %S ]" );
}

BOOST_AUTO_TEST_CASE( parametric_defined ) {
	tester t;
	t.add_defined_type( "pine", "<:T:> [:T]" );
	t( "pine<:integer:>", "prototype(pine)｢ integer ｣" );
	t( "<:Q:> pine<:Q:>", "｢ Q : ｣ prototype(pine)｢ %Q ｣");
	
	t.add_type_alias( "repine", "<:X:> pine<:X:>" );
	t( "repine<:integer:>", "prototype(pine)｢ integer ｣" );
}

BOOST_AUTO_TEST_SUITE_END()