/**
	Tests get_spec specifically. "type_identifier_determine" also tests it via a round-trip test. 
	
	Don't add many here since so much of the output is implicit.
*/
#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_identifier_get_spec )

struct tester : public type_converter_tester {
	bool allow_param;
	
	tester() :
		allow_param(false) {
	}
	
	void operator()( char const * spec, char const * expect ) {
		expect_test ex;
		ex.add( "Spec", spec );
		
		unsigned flags = allow_param ? type_identifier::df_allow_param : 0;
		type_identifier::resolve_spec_t q;
		try {
			auto t = determine_type(spec, flags);
			q = type_identifier::get_spec( the_context(), t );
		} catch(...) {
			ex.except();
		}
		
		ex.check( q.okay(), q.explain_error() );
		ex.check_actual( dump::get(q.result), ex.platform_fix(expect) );
	} 
};

BOOST_AUTO_TEST_CASE( basic ) {
	tester t;
	t( "integer", "integer !literal !lossy !lvalue !optional 64bit value" );
}

BOOST_AUTO_TEST_CASE( params ) {
	tester t;
	
	auto cl = t.add_class( "cone" );
	cl.type->params.push_back( { "T" } );

	t( "cone<:integer:>", 
		"prototype(cone)｢ integer !literal !lossy !lvalue !optional %PBITbit value ｣ !literal !lossy !lvalue !optional value" );
}

BOOST_AUTO_TEST_CASE( array ) {
	tester t;
	
	t( "array<:integer:>", 
		"prototype(array)｢ integer !literal !lossy !lvalue !optional %PBITbit value ｣ !literal !lossy !lvalue !optional value" );
}

BOOST_AUTO_TEST_CASE( value_list_refer ) {
	tester t;
	t.allow_param = true;
	
	t( "( x : array<:any:>, : type_infer(x#0))", 
		"( x : prototype(array)｢ any ｣, : type_infer( x#(0/1) ) ) !literal !lossy !lvalue !optional value" );
}

BOOST_AUTO_TEST_SUITE_END()