#include <boost/test/unit_test.hpp>

#include "util/unicode.hpp"

BOOST_AUTO_TEST_SUITE( unicode_test )

namespace u = unicode;

BOOST_AUTO_TEST_CASE( len8 )
{
	auto clen = []( size_t ulen, size_t rlen, std::string const & str ) {
		BOOST_CHECK_EQUAL( rlen, str.size() ); //ensure input is as expected
		BOOST_CHECK_EQUAL( ulen, u::len8(str) );
	};

	clen(1,1,"a");
	clen(1,2,"©");
	clen(1,2,"Ѡ");
	clen(1,3,"ஆ");
	clen(3,5,"aォb");
	clen(3,9,"🝂/🝌" ); //BORAX CALX
}

BOOST_AUTO_TEST_CASE( decode )
{
	auto cmp = []( std::string const & in, u::codestr const & expect ) {
		u::codestr out = u::decode8(in);
		BOOST_REQUIRE_EQUAL( out.size(), expect.size() );
		for( size_t i=0; i < out.size(); ++i ) {
			BOOST_CHECK_EQUAL( out[i], expect[i] );
		}
		
		//check encode as well
		auto enc = u::encode8(expect);
		BOOST_CHECK_EQUAL( in, enc );
	};
	
	cmp( "W", { 0x57 } );
	cmp( "®", { 0xAE } );
	cmp( "æ ঽ\t", { 0xE6, 0x20, 0x09BD, 0x09 } );
	cmp( "‰/✇𐀂a", { 0x2030, 0x2F, 0x2707, 0x10002 /*linearB I*/, 0x61 } );
}

BOOST_AUTO_TEST_SUITE_END()
