#include "util/dump_base.hpp"

namespace leaf_util
{

dump_base::dump_base( std::ostream & out )
	: backing( nullptr )
	, out( out )
{
	flat = false;
}

dump_base::dump_base( )
	: backing( new std::ostringstream() )
	, out( *backing )
{
	flat = false;
}

dump_base::~dump_base()
{
	delete backing;
}

void dump_base::set_flat()
{
	flat = true;
	indent_bits.clear();
}

std::string dump_base::str()
{
	PRE_CHECK( backing );
	return backing->str();
}

void dump_base::indent( char const * bit, char const * first_bit )
{
	if( flat )
		return;
		
	indent_item ii;
	ii.bit = bit;
	if( first_bit )
		ii.first_bit = first_bit;
	ii.shown_first = false;

	indent_bits.push_back( ii );
}

void dump_base::unindent( char const * end_bit )
{
	if( flat )
		return;
		
	STATE_CHECK( indent_bits.size() );
	auto last = indent_bits.back();
	indent_bits.pop_back();
	
	if( end_bit ) {
		out << indent_str();
		if( !last.shown_first && last.first_bit )
			out << *last.first_bit;
		out << end_bit << endl();
	}
}

std::string dump_base::indent_str()
{
	std::string result;
	for( auto & ii : indent_bits ) {
		if( !ii.shown_first && ii.first_bit )
			result += *ii.first_bit;
		else
			result += ii.bit;
		ii.shown_first = true;
	}

	return result;
}

void dump_base::error( std::string const & msg )
{
	out << "\e[31m" << msg << "\e[0m";
}

void dump_base::header( std::string const & msg )
{
	out << "\e[32m" << msg << "\e[0m" << std::endl;
}

} //eon leaf_util